/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "BitStream.h"

#include <string.h>

cBitStream::cBitStream(unsigned char * data, int length, bool copy)
:	copied(copy),
	length(length),
	bytePos(0),
	bitPos(0)
{
	if (copy)
	{
		this->data = new unsigned char[length];
		memcpy(this->data, data, length);
	}
	else
	{
		this->data = data;
	}
}

cBitStream::~cBitStream()
{
	if (copied)
	{
		delete[] data;
	}
}

void cBitStream::ByteAlign()
{
	if (bytePos < length)
	{
		if (bitPos > 0)
		{
			bytePos++;
			bitPos = 0;
		}
	}
}

void cBitStream::SeekStart()
{
	bytePos = 0;
	bitPos = 0;
}

void cBitStream::SeekBits(int bits)
{
	if (bytePos < length)
	{
		bitPos += bits;
		while (bitPos > 7)
		{
			bitPos -= 8;
			bytePos++;
		}
		if (bytePos > length)
		{
			bytePos = length;
			bitPos = 0;
		}
	}
}

void cBitStream::SeekBytes(int bytes)
{
	if (bytePos < length)
	{
		bytePos += bytes;
		if (bytePos >= length)
		{
			bytePos = length;
			bitPos = 0;
		}
	}
}

unsigned int cBitStream::NextBits(int bits)
{
	int oldBytePos = bytePos;
	int oldBitPos = bitPos;
	unsigned int value = GetBits(bits);
	bytePos = oldBytePos;
	bitPos = oldBitPos;
	return value;
}

unsigned int cBitStream::GetBits(int bits)
{
	unsigned int value = 0;

	while (bits > 1)
	{
		value |= GetBit();
		value <<= 1;
		bits--;
	}
	value |= GetBit();
	return value;
}

unsigned int cBitStream::GetBit()
{
	if (bytePos < length)
	{
		unsigned int bit;
		bit = (data[bytePos] >> (7 - bitPos)) & 1;
		bitPos++;
		if (bitPos > 7)
		{
			bitPos = 0;
			bytePos++;
		}
		return bit;
	}
	return 0;
}
