/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "DlgNetwork.h"
#include "DlgNetworkGame.h"
#include "DlgNetworkGeneral.h"
#include "QFinder.h"
#include "Game.h"

DlgNetwork *gDlgNetwork;

INT_PTR CALLBACK NetworkDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

DlgNetwork::DlgNetwork(HWND hParent)
{
	gDlgNetwork = this;
	fParent = hParent;
	gDlgNetworkGame = new DlgNetworkGame(this, IDD_NETWORK_GAME);
	gDlgNetworkGeneral = new DlgNetworkGeneral(this, IDD_NETWORK_GENERAL);
}

DlgNetwork::~DlgNetwork()
{
	delete gDlgNetworkGame;
	delete gDlgNetworkGeneral;
}

void DlgNetwork::Create()
{
	DialogBox(gQFinder->fInstance, MAKEINTRESOURCE(IDD_NETWORK), fParent, (DLGPROC) NetworkDialogProc);
}

INT_PTR DlgNetwork::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	NMTREEVIEW *treeView;
	TREENODE *node;

	switch (message)
	{
	case WM_INITDIALOG:
		OnInitDialog(hDlg);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			fLastFrame->Hide();
		case IDCANCEL:
			EndDialog(hDlg, wParam);
			return TRUE;
		}
		break;
	case WM_NOTIFY:
		switch (LOWORD(wParam))
		{
		case IDC_TREEVIEW:
			treeView = (NMTREEVIEW *) lParam;
			switch (treeView->hdr.code)
			{
			case TVN_SELCHANGED:
				node = (TREENODE *) treeView->itemNew.lParam;
				SetDlgItemText (hDlg, IDC_CATEGORY, node->fNodeName.c_str());
				switch (node->fDialogID)
				{
				case IDD_NETWORK_GAME:
					fLastFrame->Hide();
					fLastFrame = gDlgNetworkGame;
					gDlgNetworkGame->Show(node->fParam);
					break;
				case IDD_NETWORK_GENERAL:
					fLastFrame->Hide();
					fLastFrame = gDlgNetworkGeneral;
					gDlgNetworkGeneral->Show();
					break;
				}
				break;
			}
			break;
		}
		break;
	}
	return FALSE;
}

void DlgNetwork::OnInitDialog(HWND hDlg)
{
	RECT rect;
	int i;
	int pos;
	TREENODE *node;

	fWnd = hDlg;

	GetClientRect(hDlg, &rect);
	fTreeView.Get(fWnd, IDC_TREEVIEW);
/*
	node = new TREENODE;
	node->fNodeName = "General";
	node->fDialogID = IDD_NETWORK_GENERAL;
	node->fParam = 0;
	fTreeView.AddItem(NULL, "General", 0, (LPARAM) node);
*/
	for (i = 1; i <= kGameCount; i++)
	{
		pos = kGameSortAlph[i];
		node = new TREENODE;
		node->fNodeName = gGames[pos].fTitle;
		node->fDialogID = IDD_NETWORK_GAME;
		node->fParam = pos;
		fTreeView.AddItem(NULL, (char *) gGames[pos].fTitle.c_str(), 0, (LPARAM) node);
	}

	gDlgNetworkGame->Create();
	gDlgNetworkGeneral->Create();
	fLastFrame = gDlgNetworkGame;
	HTREEITEM hItem = fTreeView.GetFirstItem();
	if (hItem != NULL)
	{
		TVITEM item;
		fTreeView.SelectItem(hItem);
		item.hItem = hItem;
		item.mask = TVIF_PARAM;
		fTreeView.GetItem(&item);
		node = (TREENODE *) item.lParam;
		SetDlgItemText (hDlg, IDC_CATEGORY, node->fNodeName.c_str());
		fLastFrame = gDlgNetworkGame;
		gDlgNetworkGame->Show(node->fParam);
	}
}

INT_PTR CALLBACK NetworkDialogProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gDlgNetwork->DialogProc (hDlg, message, wParam, lParam);
}
