/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "DlgNetwork.h"
#include "DlgNetworkGame.h"
#include "QFinder.h"

#include "ServerList.h"
#include "GameServer.h"
#include "Utilities.h"
#include "Game.h"

DlgNetworkGame *gDlgNetworkGame;
int gColumnSort;
int gColumn;

INT_PTR CALLBACK NetworkGameDialogProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
int CALLBACK ServerList_CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);

DlgNetworkGame::DlgNetworkGame(DlgNetwork *dlg, long frameID)
:DlgNetworkFrame(dlg, frameID)
{
}

DlgNetworkGame::~DlgNetworkGame()
{
	DlgNetworkFrame::~DlgNetworkFrame();
}

void DlgNetworkGame::Create()
{
	DlgNetworkFrame::Create((DLGPROC) NetworkGameDialogProc);

	fBCPort.Get(fFrame, IDC_BCPORT);
	fBCPortUpDown.Get(fFrame, IDC_BCPORTUPDN);
	fBCPortList.Get(fFrame, IDC_BCPORTLIST);

	fIPAddress.Get(fFrame, IDC_IPADDRESS);
	fPortUpDown.Get(fFrame, IDC_PORTUPDN);
	fPort.Get(fFrame, IDC_PORT);
	fDescription.Get(fFrame, IDC_DESCRIPTION);
	fList.Get(fFrame, IDC_SERVERLIST);
	fList.SetStyleFullRowSelect();

	fBCPortUpDown.SetRange(0, 65536);

	fList.AddColumn(0, "IP", LVCFMT_LEFT, 120, 0);
	fList.AddColumn(1, "Queryport", LVCFMT_LEFT, 60, 1);
	fList.AddColumn(2, "Description", LVCFMT_LEFT, 200, 2);

	fPortUpDown.SetRange(0, 65535);
}

void DlgNetworkGame::Show(long game)
{
	fGame = game;
	DlgNetworkFrame::Show();
}

INT_PTR DlgNetworkGame::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowPos(hDlg, HWND_TOP, 170, 32, 0, 0, SWP_NOSIZE);
		return TRUE;
	case WM_SHOWWINDOW:
		if (wParam)
		{
			// is shown
			fBCPortList.ResetContent();
			set<BCPort>::iterator itPort;
			int index;
			for (itPort = gQFinder->fBCPorts.begin();
				 itPort != gQFinder->fBCPorts.end();
				 itPort++)
			{
				if (itPort->fGame == fGame)
				{
					index = fBCPortList.AddString(Port2String(itPort->fPort));
					fBCPortList.SetItemData(index, (unsigned int) itPort.operator ->());
				}
			}
			if (fBCPortList.GetCount() > 0)
			{
				fBCPortList.SetCurrentSelection(0);
				fBCPort.SetText(fBCPortList.GetText(0));
			}
			else
			{
				fBCPort.SetText(gGames[fGame].fStdPort);
			}

			fList.DeleteAllItems();
			set<GameServer>::iterator itServer;
			ServerList *svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
			if (svrList != NULL)
			{
				for (itServer = svrList->fServers.begin();
					 itServer != svrList->fServers.end();
					 itServer++)
				{
					if (itServer->fGame == fGame)
						AddServer(itServer.operator ->());
				}
			}
			fList.SetSelectedItem(0);
			LVITEM item;
			item.iItem = 0;
			item.iSubItem = 0;
			item.mask = LVIF_IMAGE | LVIF_PARAM;
			if (fList.GetItem(&item))
			{
				GameServer *server = (GameServer *) item.lParam;
				fIPAddress.SetAddress(server->fIP);
				fPort.SetText(server->fQueryPort);
				fDescription.SetText(server->fDescription);
			}
			else
			{
				fIPAddress.SetAddress(0);
				fPort.SetText(gGames[fGame].fStdPort);
				fDescription.SetText("");
			}
			gColumn = 0;
			gColumnSort = kColumnSortAscending;
			fList.SortItems(ServerList_CompareFunc, gColumn);
		}
		else
		{
			// is hidden
		}
		break;
	case WM_NOTIFY:
		return OnNotify(wParam, lParam);
	case WM_COMMAND:
		OnCommand(wParam, lParam);
		break;
	}
	return FALSE;
}

LRESULT DlgNetworkGame::OnNotify(WPARAM wParam, LPARAM lParam)
{
	NMLISTVIEW *nmListView;

	switch (LOWORD(wParam))
	{
	case IDC_SERVERLIST:
		nmListView = (NMLISTVIEW *) lParam;
		switch (nmListView->hdr.code)
		{
		case LVN_COLUMNCLICK:
			if (gColumn != nmListView->iSubItem)
				gColumnSort = kColumnSortAscending;
			else
				if (gColumnSort == kColumnSortAscending)
					gColumnSort = kColumnSortDescending;
				else
					gColumnSort = kColumnSortAscending;

			gColumn = nmListView->iSubItem;
			fList.SortItems(ServerList_CompareFunc, gColumn);
			break;
		case LVN_ITEMCHANGED:
			if (nmListView->uNewState == 3)
			{
				LVITEM item;
				item.iItem = nmListView->iItem;
				item.iSubItem = 0;
				item.mask = LVIF_IMAGE | LVIF_PARAM;
				if (fList.GetItem(&item))
				{
					GameServer *server = (GameServer *) item.lParam;
					fIPAddress.SetAddress(server->fIP);
					fPort.SetText(server->fQueryPort);
					fDescription.SetText(server->fDescription);
				}
			}
			break;
		}
		break;
	}
	return 0;
}

LRESULT DlgNetworkGame::OnCommand(WPARAM wParam, LPARAM lParam)
{
	LVITEM item;
	ServerList *svrList;
	GameServer *server;
	set<GameServer>::iterator itServer;
	set<BCPort>::iterator itPort;
	char str[100];
	int index;
	int count;
	BCPort *port;

	switch (HIWORD(wParam))
	{
	case EN_CHANGE:
		switch (LOWORD(wParam))
		{
			case IDC_BCPORT:
				fBCPortUpDown.SetPosition(fBCPort.GetInt());
				break;
			case IDC_PORT:
				fPortUpDown.SetPosition(fPort.GetInt());
				break;
		}
		break;
	case LBN_SELCHANGE:
		switch (LOWORD(wParam))
		{
			case IDC_BCPORTLIST:
				fBCPort.SetText(fBCPortList.GetText(fBCPortList.GetCurrentSelection()));
				break;
		}
		break;
	case BN_CLICKED:
		svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
		svrList->fHasChanged = true;
		switch (LOWORD(wParam))
		{
			case IDC_BCPORTADD:
				port = new BCPort(fGame, fBCPortUpDown.GetPosition());
				if (gQFinder->fBCPorts.find(*port) == gQFinder->fBCPorts.end())
				{
					itPort = gQFinder->fBCPorts.insert(gQFinder->fBCPorts.end(), *port);
					index = fBCPortList.AddString(Port2String(port->fPort));
					fBCPortList.SetItemData(index, (unsigned int) itPort.operator ->());
					fBCPortList.SetCurrentSelection(index);
				}
				delete port;
				break;
			case IDC_BCPORTREMOVE:
				if ((index = fBCPortList.GetCurrentSelection()) != LB_ERR)
				{
					port = (BCPort *) fBCPortList.GetItemData(index);
					gQFinder->fBCPorts.erase(*port);
					fBCPortList.DeleteString(index);
					if (fBCPortList.GetCount() > 0)
					{
						fBCPortList.SetCurrentSelection(0);
						fBCPort.SetText(fBCPortList.GetText(0));
					}
				}
				break;
			case IDC_FAVORADD:
				svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
				if (svrList == NULL)
					break;
				server = svrList->FindServer(fGame, fIPAddress.GetAddress(),
											 fPortUpDown.GetPosition());
				if (server != NULL)
					break;
				server = new GameServer(fIPAddress.GetAddress(), fPortUpDown.GetPosition());
				server->fGame = fGame;
				server->fDescription = fDescription.GetText();
				itServer = svrList->fServers.insert(svrList->fServers.end(), *server);
				AddServer(itServer.operator ->());
				fList.SortItems(ServerList_CompareFunc, gColumn);

				count = fList.GetItemCount();
				for (index = 0; index < count; index++)
				{
					LVITEM item;

					item.iItem = index;
					item.iSubItem = 0;
					item.mask = LVIF_IMAGE | LVIF_PARAM;
					if (fList.GetItem(&item))
					{
						GameServer *paramServer = (GameServer *) item.lParam;
						if (paramServer->fIP == itServer->fIP &&
							paramServer->fQueryPort == itServer->fQueryPort &&
							paramServer->fGame == itServer->fGame)
						{
							fList.SetSelectedItem(index);
							break;
						}
					}
				}
				delete server;
				break;
			case IDC_FAVORCHANGE:
				item.mask = LVIF_PARAM;
				if (fList.GetSelectedItem(&item))
				{
					string ip = "", port = "", desc = "";
					ip = IP2String(fIPAddress.GetAddress());
					port = fPort.GetText();
					desc = fDescription.GetText();
					server = (GameServer *) item.lParam;
					sprintf(str, "Change server\n   %s %s:%s\nto\n   %s %s:%s?",
							server->fDescription.c_str(), server->fIPStr.c_str(), server->fQueryPortStr.c_str(),
							desc.c_str(), ip.c_str(), port.c_str());
					if (MessageBox(fFrame, str, "", MB_YESNO | MB_ICONQUESTION) == IDNO)
						break;
					server->SetAddress(fIPAddress.GetAddress(), fPortUpDown.GetPosition());
					server->fDescription = desc;
					fList.DeleteItem(item.iItem);
					AddServer(server);
					fList.SortItems(ServerList_CompareFunc, gColumn);

					count = fList.GetItemCount();
					for (index = 0; index < count; index++)
					{
						LVITEM item;

						item.iItem = index;
						item.iSubItem = 0;
						item.mask = LVIF_IMAGE | LVIF_PARAM;
						if (fList.GetItem(&item))
						{
							GameServer *paramServer = (GameServer *) item.lParam;
							if (paramServer->fIP == server->fIP &&
								paramServer->fQueryPort == server->fQueryPort &&
								paramServer->fGame == server->fGame)
							{
								fList.SetSelectedItem(index);
								break;
							}
						}
					}
				}
				break;
			case IDC_FAVORREMOVE:
				item.mask = LVIF_PARAM;
				if (fList.GetSelectedItem(&item))
				{
					server = (GameServer *) item.lParam;
					sprintf(str, "Delete server %s %s:%s?", server->fDescription.c_str(),
						    server->fIPStr.c_str(), server->fQueryPortStr.c_str());
					if (MessageBox(fFrame, str, "", MB_YESNO | MB_ICONQUESTION) == IDNO)
						break;
					svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
					if (svrList == NULL)
						break;
					svrList->fServers.erase(*server);
					fList.DeleteItem(item.iItem);
				}
				break;
		}
		break;
	}
	return 0;
}

void DlgNetworkGame::AddServer(GameServer *server)
{
	int item = fList.AddItem(server->fIPStr, server->fGame, (LPARAM) server);
	fList.AddSubItem(server->fQueryPortStr, item, 1);
	fList.AddSubItem(server->fDescription, item, 2);
}


INT_PTR CALLBACK NetworkGameDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gDlgNetworkGame->DialogProc(hDlg, message, wParam, lParam);
}

int CALLBACK ServerList_CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	GameServer *server1 = (GameServer *) lParam1;
	GameServer *server2 = (GameServer *) lParam2;
	long subItem = lParamSort;
	int retVal = 0;

	switch (subItem)
	{
	case 0:
		if (server1->fIP < server2->fIP)
			retVal = -1;
		else
			if (server1->fIP > server2->fIP)
				retVal = 1;
			else
				if (server1->fQueryPort < server2->fQueryPort)
					retVal = -1;
				else
					if (server1->fQueryPort > server2->fQueryPort)
						retVal = 1;
					else
						retVal = 0;
		break;
	case 1:
		if (server1->fQueryPort < server2->fQueryPort)
			retVal = -1;
		else
			if (server1->fQueryPort > server2->fQueryPort)
				retVal = 1;
			else
				retVal = 0;
		break;
	case 2:
		retVal = server1->fDescription.compare(server2->fDescription);
		break;
	}
	if (gColumnSort == kColumnSortAscending)
		return retVal;
	else
		return -retVal;
}
