/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "DlgPreferences.h"
#include "DlgPreferencesAppearance.h"
#include "DlgPreferencesLayout.h"
#include "DlgPreferencesListView.h"
#include "DlgPreferencesHotKeys.h"
#include "DlgPreferencesGames.h"
#include "DlgPreferencesGame.h"
#include "QFinder.h"
#include "Game.h"

DlgPreferences *gDlgPreferences;

INT_PTR CALLBACK PreferencesDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

DlgPreferences::DlgPreferences(HWND hParent)
{
	gDlgPreferences = this;
	fParent = hParent;
	gDlgPreferencesAppearance = new DlgPreferencesAppearance(this, IDD_PREFERENCES_APPEARANCE);
	gDlgPreferencesLayout = new DlgPreferencesLayout(this, IDD_PREFERENCES_LAYOUT);
	gDlgPreferencesListView = new DlgPreferencesListView(this, IDD_PREFERENCES_LISTVIEW);
	gDlgPreferencesHotKeys = new DlgPreferencesHotKeys(this, IDD_PREFERENCES_HOTKEYS);
	gDlgPreferencesGames = new DlgPreferencesGames(this, IDD_PREFERENCES_GAMES);
	gDlgPreferencesGame = new DlgPreferencesGame(this, IDD_PREFERENCES_GAME);
}

DlgPreferences::~DlgPreferences()
{
	delete gDlgPreferencesAppearance;
	delete gDlgPreferencesLayout;
	delete gDlgPreferencesListView;
	delete gDlgPreferencesHotKeys;
	delete gDlgPreferencesGames;
	delete gDlgPreferencesGame;
}

void DlgPreferences::Create()
{
	DialogBox(gQFinder->fInstance, MAKEINTRESOURCE(IDD_PREFERENCES), fParent, (DLGPROC) PreferencesDialogProc);
}

INT_PTR DlgPreferences::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	NMTREEVIEW *treeView;
	TREENODE *node;

	switch (message)
	{
	case WM_INITDIALOG:
		OnInitDialog(hDlg);
		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			fLastFrame->Hide();
			fSettings.SetChanged(gQFinder->fSettings);
			gQFinder->fSettings = fSettings;
		case IDCANCEL:
			EndDialog(hDlg, wParam);
			return TRUE;
		}
		break;
	case WM_NOTIFY:
		switch (LOWORD(wParam))
		{
		case IDC_TREEVIEW:
			treeView = (NMTREEVIEW *) lParam;
			switch (treeView->hdr.code)
			{
			case TVN_SELCHANGED:
				node = (TREENODE *) treeView->itemNew.lParam;
				SetDlgItemText (hDlg, IDC_CATEGORY, node->fNodeName.c_str());
				switch (node->fDialogID)
				{
				case IDD_PREFERENCES_APPEARANCE:
					fLastFrame->Hide();
					fLastFrame = gDlgPreferencesAppearance;
					gDlgPreferencesAppearance->Show();
					break;
				case IDD_PREFERENCES_LAYOUT:
					fLastFrame->Hide();
					fLastFrame = gDlgPreferencesLayout;
					gDlgPreferencesLayout->Show();
					break;
				case IDD_PREFERENCES_LISTVIEW:
					fLastFrame->Hide();
					fLastFrame = gDlgPreferencesListView;
					gDlgPreferencesListView->Show(node->fParam);
					break;
				case IDD_PREFERENCES_HOTKEYS:
					fLastFrame->Hide();
					fLastFrame = gDlgPreferencesHotKeys;
					gDlgPreferencesHotKeys->Show();
					break;
				case IDD_PREFERENCES_GAMES:
					fLastFrame->Hide();
					fLastFrame = gDlgPreferencesGames;
					gDlgPreferencesGames->Show();
					break;
				case IDD_PREFERENCES_GAME:
					fLastFrame->Hide();
					fLastFrame = gDlgPreferencesGame;
					gDlgPreferencesGame->Show (node->fParam);
					break;
				}
				break;
			}
			break;
		}
		break;
	}
	return FALSE;
}

void DlgPreferences::OnInitDialog(HWND hDlg)
{
	RECT rect;
	HTREEITEM hItem;
	int i;
	int pos;
	TREENODE *node;

	fWnd = hDlg;

	fSettings = gQFinder->fSettings;
	gQFinder->fSettings.fChanged = 0;

	GetClientRect(hDlg, &rect);
	fTreeView.Get(fWnd, IDC_TREEVIEW);

	node = new TREENODE;
	node->fNodeName = "Appearance";
	node->fDialogID = IDD_PREFERENCES_APPEARANCE;
	node->fParam = 0;
	hItem = fTreeView.AddItem(NULL, "Appearance", 0, (LPARAM) node);

	node = new TREENODE;
	node->fNodeName = "Layout";
	node->fDialogID = IDD_PREFERENCES_LAYOUT;
	node->fParam = 0;
	fTreeView.AddItem(hItem, "Layout", 0, (LPARAM) node);

	node = new TREENODE;
	node->fNodeName = "Server List";
	node->fDialogID = IDD_PREFERENCES_LISTVIEW;
	node->fParam = IDC_SERVER_LISTVIEW;
	fTreeView.AddItem(hItem, "Server List", 0, (LPARAM) node);

	node = new TREENODE;
	node->fNodeName = "Player List";
	node->fDialogID = IDD_PREFERENCES_LISTVIEW;
	node->fParam = IDC_PLAYER_LISTVIEW;
	fTreeView.AddItem(hItem, "Player List", 0, (LPARAM) node);

	node = new TREENODE;
	node->fNodeName = "Variable List";
	node->fDialogID = IDD_PREFERENCES_LISTVIEW;
	node->fParam = IDC_VARIABLE_LISTVIEW;
	fTreeView.AddItem(hItem, "Variable List", 0, (LPARAM) node);

	fTreeView.Expand(hItem, TVE_EXPAND);

	node = new TREENODE;
	node->fNodeName = "Hotkeys";
	node->fDialogID = IDD_PREFERENCES_HOTKEYS;
	node->fParam = 0;
	hItem = fTreeView.AddItem(NULL, "Hotkeys", 0, (LPARAM) node);

	node = new TREENODE;
	node->fNodeName = "Games";
	node->fDialogID = IDD_PREFERENCES_GAMES;
	node->fParam = 0;
	hItem = fTreeView.AddItem(NULL, "Games", 0, (LPARAM) node);

	for (i = 1; i < (int) gGames.size(); i++)
	{
		pos = kGameSortAlph[i];
		node = new TREENODE;
		node->fNodeName = gGames[pos].fTitle;
		node->fDialogID = IDD_PREFERENCES_GAME;
		node->fParam = pos;
		fTreeView.AddItem(hItem, (char *) gGames[pos].fTitle.c_str (), 0, (LPARAM) node);
	}

	fTreeView.Expand(hItem, TVE_EXPAND);

	gDlgPreferencesAppearance->Create();
	gDlgPreferencesLayout->Create();
	gDlgPreferencesListView->Create();
	gDlgPreferencesHotKeys->Create();
	gDlgPreferencesGames->Create();
	gDlgPreferencesGame->Create();
	fLastFrame = gDlgPreferencesAppearance;
	hItem = fTreeView.GetFirstItem();
	if (hItem != NULL)
	{
		TVITEM item;
		fTreeView.SelectItem(hItem);
		item.hItem = hItem;
		item.mask = TVIF_PARAM;
		fTreeView.GetItem(&item);
		node = (TREENODE *) item.lParam;
		SetDlgItemText (hDlg, IDC_CATEGORY, node->fNodeName.c_str());
		fLastFrame = gDlgPreferencesAppearance;
		gDlgPreferencesAppearance->Show();
	}
}

INT_PTR CALLBACK PreferencesDialogProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gDlgPreferences->DialogProc (hDlg, message, wParam, lParam);
}
