/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "DlgPreferences.h"
#include "DlgPreferencesGame.h"
#include "QFinder.h"
#include "Game.h"

DlgPreferencesGame *gDlgPreferencesGame;

INT_PTR CALLBACK PreferencesGameDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

DlgPreferencesGame::DlgPreferencesGame(DlgPreferences *dlg, long frameID)
:DlgPreferencesFrame(dlg, frameID)
{
}

DlgPreferencesGame::~DlgPreferencesGame()
{
	DlgPreferencesFrame::~DlgPreferencesFrame();
}

void DlgPreferencesGame::Create()
{
	DlgPreferencesFrame::Create((DLGPROC) PreferencesGameDialogProc);
}

void DlgPreferencesGame::Show(long game)
{
	fGame = game;
	DlgPreferencesFrame::Show();
}

INT_PTR DlgPreferencesGame::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowPos(hDlg, HWND_TOP, 226, 32, 0, 0, SWP_NOSIZE);
		fEditFolder.Get(hDlg, IDC_EDIT_FOLDER);
		fEditExe.Get(hDlg, IDC_EDIT_EXE);
		fEditParam.Get(hDlg, IDC_EDIT_PARAMS);
		return TRUE;
	case WM_SHOWWINDOW:
		if (wParam)
		{
			// is shown
			fEditFolder.SetText(fDlg->fSettings.fGame.fInfos[fGame].fPath);
			fEditExe.SetText(fDlg->fSettings.fGame.fInfos[fGame].fExe);
			fEditParam.SetText(fDlg->fSettings.fGame.fInfos[fGame].fParameters);
		}
		else
		{
			// is hidden
			fDlg->fSettings.fGame.fInfos[fGame].fPath = fEditFolder.GetText();
			fDlg->fSettings.fGame.fInfos[fGame].fExe = fEditExe.GetText();
			fDlg->fSettings.fGame.fInfos[fGame].fParameters = fEditParam.GetText();
		}
		break;
	case WM_COMMAND:
		switch (HIWORD(wParam))
		{
		case BN_CLICKED:
			switch (LOWORD(wParam))
			{
			case IDC_BTN_BROWSE:
				char szPath[1001];
				string path;
				string file;
				string title;
				OPENFILENAME ofn;

				path = fEditFolder.GetText();
				file = fEditExe.GetText();
				if (path.length() == 0)
				{
					path = file;
				}
				else
				{
					if (path[path.length() - 1] != '\\')
					{
						path += "\\";
					}
					path += file;
				}
				strcpy(szPath, path.c_str());
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize = sizeof (OPENFILENAME);
				ofn.hwndOwner = fFrame;
				ofn.lpstrFilter = "Executable Files\0*.exe;*.bat\0All Files (*.*)\0*.*\0\0";
				ofn.nFilterIndex = 1;
				ofn.lpstrFile = szPath;
				ofn.nMaxFile = 1000;
				title = "Choose " + gGames[fGame].fTitle + " executable...";
				ofn.lpstrTitle = title.c_str();
				ofn.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR;
				if (!GetOpenFileName(&ofn))
				{
					DWORD dwErr = CommDlgExtendedError();
				}
				else
				{
					path = ofn.lpstrFile;
					long pos = path.rfind('\\');
					if (pos != -1)
					{
						file = path.substr(pos + 1);
						path = path.substr(0, pos + 1);
						fEditFolder.SetText(path);
						fEditExe.SetText(file);
					}
				}
				break;
			}
			break;
		}
		break;
	}
	return FALSE;
}


INT_PTR CALLBACK PreferencesGameDialogProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gDlgPreferencesGame->DialogProc (hDlg, message, wParam, lParam);
}
