/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "DlgPreferences.h"
#include "DlgPreferencesGames.h"
#include "QFinder.h"

DlgPreferencesGames *gDlgPreferencesGames;

INT_PTR CALLBACK PreferencesGamesDialogProc(HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam);

DlgPreferencesGames::DlgPreferencesGames(DlgPreferences *dlg, long frameID)
:DlgPreferencesFrame (dlg, frameID)
{
}

DlgPreferencesGames::~DlgPreferencesGames()
{
	DlgPreferencesFrame::~DlgPreferencesFrame();
}

void DlgPreferencesGames::Create()
{
	HWND item;

	DlgPreferencesFrame::Create((DLGPROC) PreferencesGamesDialogProc);

	item = GetDlgItem(fFrame, IDC_UP);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_UPARROW)));
	item = GetDlgItem(fFrame, IDC_DOWN);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_DNARROW)));
	item = GetDlgItem(fFrame, IDC_TOP);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_TOPARROW)));
	item = GetDlgItem(fFrame, IDC_BOTTOM);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_BTMARROW)));
}

INT_PTR DlgPreferencesGames::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i;
	int currentItem;
	int game;
	int count;

	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowPos(hDlg, HWND_TOP, 226, 32, 0, 0, SWP_NOSIZE);
		fGameList.Get(hDlg, IDC_GAMELIST);
		return TRUE;
	case WM_SHOWWINDOW:
		if (wParam)
		{
			// is shown
			fGameList.ResetContent();
			for (i = 1; i <= kGameCount; i++)
			{
				fGameSort[i - 1] = fDlg->fSettings.fGame.fGameSort[i];
				fGameList.AddString(gGames[fGameSort[i - 1]].fTitle);
			}
		}
		else
		{
			for (i = 1; i <= kGameCount; i++)
			{
				fDlg->fSettings.fGame.fGameSort[i] = fGameSort[i - 1];
			}
		}
		break;
	case WM_COMMAND:
		switch (HIWORD(wParam))
		{
		case BN_CLICKED:
			switch (LOWORD(wParam))
			{
			case IDC_TOP:
				currentItem = fGameList.GetCurrentSelection();
				if (currentItem == LB_ERR || currentItem == 0)
					break;
				game = fGameSort[currentItem];
				for (i = currentItem; i > 0; i--)
					fGameSort[i] = fGameSort[i - 1];
				fGameSort[0] = game;
				fGameList.ResetContent();
				for (i = 0; i < kGameCount; i++)
					fGameList.AddString(gGames[fGameSort[i]].fTitle);
				fGameList.SetCurrentSelection(0);
				break;
			case IDC_BOTTOM:
				currentItem = fGameList.GetCurrentSelection();
				count = fGameList.GetCount();
				if (currentItem == LB_ERR || currentItem == count - 1)
					break;
				game = fGameSort[currentItem];
				for (i = currentItem; i < count - 1; i++)
					fGameSort[i] = fGameSort[i + 1];
				fGameSort[count - 1] = game;
				fGameList.ResetContent();
				for (i = 0; i < kGameCount; i++)
					fGameList.AddString(gGames[fGameSort[i]].fTitle);
				fGameList.SetCurrentSelection(count - 1);
				break;
			case IDC_UP:
				currentItem = fGameList.GetCurrentSelection();
				if (currentItem == LB_ERR || currentItem == 0)
					break;
				game = fGameSort[currentItem - 1];
				fGameSort[currentItem - 1] = fGameSort[currentItem];
				fGameSort[currentItem] = game;
				fGameList.ResetContent();
				for (i = 0; i < kGameCount; i++)
					fGameList.AddString(gGames[fGameSort[i]].fTitle);
				fGameList.SetCurrentSelection(currentItem - 1);
				break;
			case IDC_DOWN:
				currentItem = fGameList.GetCurrentSelection();
				if (currentItem == LB_ERR || currentItem == fGameList.GetCount() - 1)
					break;
				game = fGameSort[currentItem + 1];
				fGameSort[currentItem + 1] = fGameSort[currentItem];
				fGameSort[currentItem] = game;
				fGameList.ResetContent();
				for (i = 0; i < kGameCount; i++)
					fGameList.AddString(gGames[fGameSort[i]].fTitle);
				fGameList.SetCurrentSelection(currentItem + 1);
				break;
			}
			break;
		}
		break;
	}
	return FALSE;
}


INT_PTR CALLBACK PreferencesGamesDialogProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gDlgPreferencesGames->DialogProc (hDlg, message, wParam, lParam);
}
