/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "DlgPreferences.h"
#include "DlgPreferencesHotKeys.h"
#include "QFinder.h"

DlgPreferencesHotKeys *gDlgPreferencesHotKeys;

INT_PTR CALLBACK PreferencesHotKeysDialogProc (HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam);

DlgPreferencesHotKeys::DlgPreferencesHotKeys(DlgPreferences *dlg, long lFrameID)
:DlgPreferencesFrame (dlg, lFrameID)
{
}

DlgPreferencesHotKeys::~DlgPreferencesHotKeys()
{
	DlgPreferencesFrame::~DlgPreferencesFrame();
}

void DlgPreferencesHotKeys::Create()
{
	DlgPreferencesFrame::Create((DLGPROC) PreferencesHotKeysDialogProc);
}

INT_PTR DlgPreferencesHotKeys::DialogProc(HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch (Message)
	{
	case WM_INITDIALOG:
		SetWindowPos(hDlg, HWND_TOP, 226, 32, 0, 0, SWP_NOSIZE);
		return TRUE;
	case WM_SHOWWINDOW:
		if (wParam)
		{
			// is shown
			if (fDlg->fSettings.fHotkeys.minimizeActive)
				CheckDlgButton(fFrame, IDC_HOTKEY_MINIMIZE_ACTIVE, BST_CHECKED);
			else
				CheckDlgButton(fFrame, IDC_HOTKEY_MINIMIZE_ACTIVE, BST_UNCHECKED);

			if (fDlg->fSettings.fHotkeys.minimizeRefresh)
				CheckDlgButton(fFrame, IDC_HOTKEY_MINIMIZE_REFRESH, BST_CHECKED);
			else
				CheckDlgButton(fFrame, IDC_HOTKEY_MINIMIZE_REFRESH, BST_UNCHECKED);

			unsigned char mod = 0;
			if (fDlg->fSettings.fHotkeys.minimizeModifier & MOD_ALT)
				mod |= HOTKEYF_ALT;
			if (fDlg->fSettings.fHotkeys.minimizeModifier & MOD_CONTROL)
				mod |= HOTKEYF_CONTROL;
			if (fDlg->fSettings.fHotkeys.minimizeModifier & MOD_SHIFT)
				mod |= HOTKEYF_SHIFT;
			HWND item = GetDlgItem(fFrame, IDC_HOTKEY_MINIMIZE);
			SendMessage(item, HKM_SETHOTKEY,
				MAKEWORD(fDlg->fSettings.fHotkeys.minimizeVirtualKey, mod), 0);
		}
		else
		{
			if (IsDlgButtonChecked(fFrame, IDC_HOTKEY_MINIMIZE_ACTIVE) == BST_CHECKED)
				fDlg->fSettings.fHotkeys.minimizeActive = true;
			else
				fDlg->fSettings.fHotkeys.minimizeActive = false;

			if (IsDlgButtonChecked(fFrame, IDC_HOTKEY_MINIMIZE_REFRESH) == BST_CHECKED)
				fDlg->fSettings.fHotkeys.minimizeRefresh = true;
			else
				fDlg->fSettings.fHotkeys.minimizeRefresh = false;

			HWND item = GetDlgItem(fFrame, IDC_HOTKEY_MINIMIZE);
			unsigned short hotkey = (unsigned short) SendMessage(item, HKM_GETHOTKEY, 0, 0);
			fDlg->fSettings.fHotkeys.minimizeVirtualKey = LOBYTE(hotkey);
			unsigned char mod = HIBYTE(hotkey);
			fDlg->fSettings.fHotkeys.minimizeModifier = 0;
			if (mod & HOTKEYF_ALT)
				fDlg->fSettings.fHotkeys.minimizeModifier |= MOD_ALT;
			if (mod & HOTKEYF_CONTROL)
				fDlg->fSettings.fHotkeys.minimizeModifier |= MOD_CONTROL;
			if (mod & HOTKEYF_SHIFT)
				fDlg->fSettings.fHotkeys.minimizeModifier |= MOD_SHIFT;
		}
		break;
	case WM_COMMAND:
		switch (HIWORD (wParam))
		{
		case BN_CLICKED:
			switch (LOWORD (wParam))
			{
			case 0:;
			}
			break;
		}
		break;
	}
	return FALSE;
}


INT_PTR CALLBACK PreferencesHotKeysDialogProc (HWND hDlg, UINT Message, WPARAM wParam, LPARAM lParam)
{
	return gDlgPreferencesHotKeys->DialogProc (hDlg, Message, wParam, lParam);
}
