/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "DlgPreferences.h"
#include "DlgPreferencesLayout.h"
#include "QFinder.h"

DlgPreferencesLayout *gDlgPreferencesLayout;

INT_PTR CALLBACK PreferencesLayoutDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

DlgPreferencesLayout::DlgPreferencesLayout(DlgPreferences *dlg, long frameID)
:DlgPreferencesFrame (dlg, frameID)
{
	fLayoutStdShortLeft = LoadBitmap(gQFinder->fInstance, MAKEINTRESOURCE(IDB_LAYOUT_STD_TSHORT_PLEFT));
	fLayoutStdShortRight = LoadBitmap(gQFinder->fInstance, MAKEINTRESOURCE(IDB_LAYOUT_STD_TSHORT_PRIGHT));
	fLayoutStdLongLeft = LoadBitmap(gQFinder->fInstance, MAKEINTRESOURCE(IDB_LAYOUT_STD_TLONG_PLEFT));
	fLayoutStdLongRight = LoadBitmap(gQFinder->fInstance, MAKEINTRESOURCE(IDB_LAYOUT_STD_TLONG_PRIGHT));
}

DlgPreferencesLayout::~DlgPreferencesLayout()
{
	DeleteObject(fLayoutStdShortLeft);
	DeleteObject(fLayoutStdShortRight);
	DeleteObject(fLayoutStdLongLeft);
	DeleteObject(fLayoutStdLongRight);
	DlgPreferencesFrame::~DlgPreferencesFrame();
}

void DlgPreferencesLayout::Create()
{
	DlgPreferencesFrame::Create((DLGPROC) PreferencesLayoutDialogProc);
}

INT_PTR DlgPreferencesLayout::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowPos(hDlg, HWND_TOP, 226, 32, 0, 0, SWP_NOSIZE);
		return TRUE;
	case WM_SHOWWINDOW:
		if (wParam)
		{
			// is shown
			Refresh();
		}
		break;
	case WM_COMMAND:
		switch (HIWORD(wParam))
		{
		case BN_CLICKED:
			switch (LOWORD(wParam))
			{
			case IDC_RADIO_STANDARD:
				fDlg->fSettings.fMainWindow.fLayout = kLayout_Standard;
				break;
			case IDC_RADIO_PARAM1_1:
				fDlg->fSettings.fMainWindow.fLayoutParameter1 = kLayout_TreeViewShort;
				break;
			case IDC_RADIO_PARAM1_2:
				fDlg->fSettings.fMainWindow.fLayoutParameter1 = kLayout_TreeViewLong;
				break;
			case IDC_RADIO_PARAM2_1:
				fDlg->fSettings.fMainWindow.fLayoutParameter2 = kLayout_PlayerLeft;
				break;
			case IDC_RADIO_PARAM2_2:
				fDlg->fSettings.fMainWindow.fLayoutParameter2 = kLayout_PlayerRight;
				break;
			}
			Refresh();
			break;
		}
		break;
	}
	return FALSE;
}

void DlgPreferencesLayout::Refresh()
{
	HWND item;

	if (fDlg->fSettings.fMainWindow.fLayout == kLayout_Standard)
	{
		CheckRadioButton(fFrame, IDC_RADIO_STANDARD, IDC_RADIO_MINI, IDC_RADIO_STANDARD);
		SetDlgItemText(fFrame, IDC_PARAM1_TEXT, "Game List:");
		SetDlgItemText(fFrame, IDC_RADIO_PARAM1_1, "short");
		SetDlgItemText(fFrame, IDC_RADIO_PARAM1_2, "long");
		SetDlgItemText(fFrame, IDC_PARAM2_TEXT, "Player List:");
		SetDlgItemText(fFrame, IDC_RADIO_PARAM2_1, "left");
		SetDlgItemText(fFrame, IDC_RADIO_PARAM2_2, "right");
		if (fDlg->fSettings.fMainWindow.fLayoutParameter1 == kLayout_TreeViewShort)
		{
			CheckRadioButton(fFrame, IDC_RADIO_PARAM1_1, IDC_RADIO_PARAM1_2, IDC_RADIO_PARAM1_1);
			if (fDlg->fSettings.fMainWindow.fLayoutParameter2 == kLayout_PlayerLeft)
			{
				CheckRadioButton(fFrame, IDC_RADIO_PARAM2_1, IDC_RADIO_PARAM2_2, IDC_RADIO_PARAM2_1);
				item = GetDlgItem(fFrame, IDC_BITMAP_STANDARD);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM1_1);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM1_2);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM2_1);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM2_2);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortRight);
			}
			if (fDlg->fSettings.fMainWindow.fLayoutParameter2 == kLayout_PlayerRight)
			{
				CheckRadioButton(fFrame, IDC_RADIO_PARAM2_1, IDC_RADIO_PARAM2_2, IDC_RADIO_PARAM2_2);
				item = GetDlgItem(fFrame, IDC_BITMAP_STANDARD);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortRight);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM1_1);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortRight);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM1_2);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongRight);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM2_1);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM2_2);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortRight);
			}
		}
		if (fDlg->fSettings.fMainWindow.fLayoutParameter1 == kLayout_TreeViewLong)
		{
			CheckRadioButton(fFrame, IDC_RADIO_PARAM1_1, IDC_RADIO_PARAM1_2, IDC_RADIO_PARAM1_2);
			if (fDlg->fSettings.fMainWindow.fLayoutParameter2 == kLayout_PlayerLeft)
			{
				CheckRadioButton(fFrame, IDC_RADIO_PARAM2_1, IDC_RADIO_PARAM2_2, IDC_RADIO_PARAM2_1);
				item = GetDlgItem(fFrame, IDC_BITMAP_STANDARD);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM1_1);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM1_2);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM2_1);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM2_2);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongRight);
			}
			if (fDlg->fSettings.fMainWindow.fLayoutParameter2 == kLayout_PlayerRight)
			{
				CheckRadioButton(fFrame, IDC_RADIO_PARAM2_1, IDC_RADIO_PARAM2_2, IDC_RADIO_PARAM2_2);
				item = GetDlgItem(fFrame, IDC_BITMAP_STANDARD);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongRight);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM1_1);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdShortRight);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM1_2);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongRight);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM2_1);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongLeft);
				item = GetDlgItem(fFrame, IDC_BITMAP_PARAM2_2);
				SendMessage(item, STM_SETIMAGE, IMAGE_BITMAP, (LPARAM) fLayoutStdLongRight);
			}
		}
	}
}

INT_PTR CALLBACK PreferencesLayoutDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gDlgPreferencesLayout->DialogProc (hDlg, message, wParam, lParam);
}
