/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "DlgPreferences.h"
#include "DlgPreferencesListView.h"
#include "QFinder.h"

DlgPreferencesListView *gDlgPreferencesListView;

INT_PTR CALLBACK PreferencesListViewDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

DlgPreferencesListView::DlgPreferencesListView(DlgPreferences *dlg, long frameID)
:DlgPreferencesFrame (dlg, frameID)
{
}

DlgPreferencesListView::~DlgPreferencesListView()
{
	DlgPreferencesFrame::~DlgPreferencesFrame();
}

void DlgPreferencesListView::Create()
{
	HWND item;

	DlgPreferencesFrame::Create((DLGPROC) PreferencesListViewDialogProc);
	item = GetDlgItem(fFrame, IDC_BTN_UP);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_UPARROW)));
	item = GetDlgItem(fFrame, IDC_BTN_DOWN);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_DNARROW)));
	item = GetDlgItem(fFrame, IDC_BTN_TOP);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_TOPARROW)));
	item = GetDlgItem(fFrame, IDC_BTN_BOTTOM);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_BTMARROW)));
	item = GetDlgItem(fFrame, IDC_BTN_ADD);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_RTARROW)));
	item = GetDlgItem(fFrame, IDC_BTN_REMOVE);
	SendMessage(item, BM_SETIMAGE, IMAGE_ICON, (LPARAM) LoadIcon (gQFinder->fInstance, MAKEINTRESOURCE(IDI_LTARROW)));
}

void DlgPreferencesListView::Show(long listViewID)
{
	fListView = listViewID;
	DlgPreferencesFrame::Show();
}

INT_PTR DlgPreferencesListView::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i;
	int j;
	int currentItem;
	int count;
	COLUMNINFO column;

	switch (message)
	{
	case WM_INITDIALOG:
		SetWindowPos(hDlg, HWND_TOP, 226, 32, 0, 0, SWP_NOSIZE);
		fListAvailable.Get(hDlg, IDC_LST_AVAILABLE);
		fListSelected.Get(hDlg, IDC_LST_SELECTED);
		return TRUE;
	case WM_SHOWWINDOW:
		if (wParam)
		{
			// is shown
			fAvailable.clear();
			fSelected.clear();
			switch (fListView)
			{
			case IDC_SERVER_LISTVIEW:
				for (i = 0; i < (int) fDlg->fSettings.fMainWindow.fServerColumns.size(); i++)
				{
					if (fDlg->fSettings.fMainWindow.fServerColumns[i].fVisible)
						fSelected.push_back(fDlg->fSettings.fMainWindow.fServerColumns[i]);
					else
						fAvailable.push_back(fDlg->fSettings.fMainWindow.fServerColumns[i]);
				}
				break;
			case IDC_PLAYER_LISTVIEW:
				for (i = 0; i < (int) fDlg->fSettings.fMainWindow.fPlayerColumns.size(); i++)
				{
					if (fDlg->fSettings.fMainWindow.fPlayerColumns[i].fVisible)
						fSelected.push_back(fDlg->fSettings.fMainWindow.fPlayerColumns[i]);
					else
						fAvailable.push_back(fDlg->fSettings.fMainWindow.fPlayerColumns[i]);
				}
				break;
			case IDC_VARIABLE_LISTVIEW:
				for (i = 0; i < (int) fDlg->fSettings.fMainWindow.fVariableColumns.size(); i++)
				{
					if (fDlg->fSettings.fMainWindow.fVariableColumns[i].fVisible)
						fSelected.push_back(fDlg->fSettings.fMainWindow.fVariableColumns[i]);
					else
						fAvailable.push_back(fDlg->fSettings.fMainWindow.fVariableColumns[i]);
				}
				break;
			}
			fListAvailable.ResetContent();
			for (i = 0; i < (int) fAvailable.size(); i++)
				fListAvailable.AddString(fAvailable[i].fTitle);
			for (i = 1; i < (int) fSelected.size(); i++)
				for (j = i - 1; j >= 0; j--)
					if (fSelected[j + 1].fPosition < fSelected[j].fPosition)
					{
						column = fSelected[j];
						fSelected[j] = fSelected[j + 1];
						fSelected[j + 1] = column;
					}
					else
					{
						break;
					}
			for (i = 0; i < (int) fSelected.size(); i++)
				fSelected[i].fPosition = i;
			fListSelected.ResetContent();
			for (i = 0; i < (int) fSelected.size(); i++)
				fListSelected.AddString(fSelected[i].fTitle);
		}
		else
		{
			switch (fListView)
			{
			case IDC_SERVER_LISTVIEW:
				for (i = 0; i < (int) fSelected.size(); i++)
				{
					for (j = 0; j < (int) fDlg->fSettings.fMainWindow.fServerColumns.size(); j++)
					{
						if (fDlg->fSettings.fMainWindow.fServerColumns[j].fID == fSelected[i].fID)
						{
							fDlg->fSettings.fMainWindow.fServerColumns[j].fVisible = true;
							fDlg->fSettings.fMainWindow.fServerColumns[j].fPosition = fSelected[i].fPosition;
							break;
						}
					}
				}
				for (i = 0; i < (int) fAvailable.size(); i++)
				{
					for (j = 0; j < (int) fDlg->fSettings.fMainWindow.fServerColumns.size(); j++)
					{
						if (fDlg->fSettings.fMainWindow.fServerColumns[j].fID == fAvailable[i].fID)
						{
							fDlg->fSettings.fMainWindow.fServerColumns[j].fVisible = false;
							fDlg->fSettings.fMainWindow.fServerColumns[j].fPosition = -1;
							break;
						}
					}
				}
				break;
			case IDC_PLAYER_LISTVIEW:
				for (i = 0; i < (int) fSelected.size(); i++)
				{
					for (j = 0; j < (int) fDlg->fSettings.fMainWindow.fPlayerColumns.size(); j++)
					{
						if (fDlg->fSettings.fMainWindow.fPlayerColumns[j].fID == fSelected[i].fID)
						{
							fDlg->fSettings.fMainWindow.fPlayerColumns[j].fVisible = true;
							fDlg->fSettings.fMainWindow.fPlayerColumns[j].fPosition = fSelected[i].fPosition;
							break;
						}
					}
				}
				for (i = 0; i < (int) fAvailable.size(); i++)
				{
					for (j = 0; j < (int) fDlg->fSettings.fMainWindow.fPlayerColumns.size(); j++)
					{
						if (fDlg->fSettings.fMainWindow.fPlayerColumns[j].fID == fAvailable[i].fID)
						{
							fDlg->fSettings.fMainWindow.fPlayerColumns[j].fVisible = false;
							fDlg->fSettings.fMainWindow.fPlayerColumns[j].fPosition = -1;
							break;
						}
					}
				}
				break;
			case IDC_VARIABLE_LISTVIEW:
				for (i = 0; i < (int) fSelected.size(); i++)
				{
					for (j = 0; j < (int) fDlg->fSettings.fMainWindow.fVariableColumns.size(); j++)
					{
						if (fDlg->fSettings.fMainWindow.fVariableColumns[j].fID == fSelected[i].fID)
						{
							fDlg->fSettings.fMainWindow.fVariableColumns[j].fVisible = true;
							fDlg->fSettings.fMainWindow.fVariableColumns[j].fPosition = fSelected[i].fPosition;
							break;
						}
					}
				}
				for (i = 0; i < (int) fAvailable.size(); i++)
				{
					for (j = 0; j < (int) fDlg->fSettings.fMainWindow.fVariableColumns.size(); j++)
					{
						if (fDlg->fSettings.fMainWindow.fVariableColumns[j].fID == fAvailable[i].fID)
						{
							fDlg->fSettings.fMainWindow.fVariableColumns[j].fVisible = false;
							fDlg->fSettings.fMainWindow.fVariableColumns[j].fPosition = -1;
							break;
						}
					}
				}
				break;
			}
		}
		break;
	case WM_COMMAND:
		switch (HIWORD(wParam))
		{
		case BN_CLICKED:
			switch (LOWORD(wParam))
			{
			case IDC_BTN_ADD:
				count = fListAvailable.GetCount();
				currentItem = fListAvailable.GetCurrentSelection();
				if (currentItem == LB_ERR || count == 0)
					break;
				column = fAvailable[currentItem];
				fAvailable.erase(fAvailable.begin() + currentItem, fAvailable.begin() + currentItem + 1);
				column.fPosition = fListSelected.GetCount();
				fSelected.push_back(column);
				fListAvailable.ResetContent();
				for (i = 0; i < (int) fAvailable.size(); i++)
					fListAvailable.AddString(fAvailable[i].fTitle);
				fListSelected.ResetContent();
				for (i = 0; i < (int) fSelected.size(); i++)
					fListSelected.AddString(fSelected[i].fTitle);
				count = fListAvailable.GetCount();
				if (count > 0)
				{
					currentItem = currentItem < count ? currentItem : count - 1;
					fListAvailable.SetCurrentSelection(currentItem);
				}
				break;
			case IDC_BTN_REMOVE:
				count = fListSelected.GetCount();
				if (count == 1)
					break;
				currentItem = fListSelected.GetCurrentSelection();
				if (currentItem == LB_ERR)
					break;
				column = fSelected[currentItem];
				fSelected.erase(fSelected.begin() + currentItem, fSelected.begin() + currentItem + 1);
				column.fPosition = -1;
				fAvailable.push_back(column);
				fListAvailable.ResetContent();
				for (i = 0; i < (int) fAvailable.size(); i++)
					fListAvailable.AddString(fAvailable[i].fTitle);
				fListSelected.ResetContent();
				for (i = 0; i < (int) fSelected.size(); i++)
					fListSelected.AddString(fSelected[i].fTitle);
				count = fListSelected.GetCount();
				if (count > 0)
				{
					currentItem = currentItem < count ? currentItem : count - 1;
					fListSelected.SetCurrentSelection(currentItem);
				}
				break;
			case IDC_BTN_TOP:
				currentItem = fListSelected.GetCurrentSelection();
				if (currentItem == LB_ERR || currentItem == 0)
					break;
				column = fSelected[currentItem];
				for (i = currentItem; i > 0; i--)
				{
					fSelected[i] = fSelected[i - 1];
					fSelected[i].fPosition = i;
				}
				fSelected[0] = column;
				fSelected[0].fPosition = 0;
				fListSelected.ResetContent();
				for (i = 0; i < (int) fSelected.size(); i++)
					fListSelected.AddString(fSelected[i].fTitle);
				fListSelected.SetCurrentSelection(0);
				break;
			case IDC_BTN_BOTTOM:
				currentItem = fListSelected.GetCurrentSelection();
				count = fListSelected.GetCount();
				if (currentItem == LB_ERR || currentItem == count - 1)
					break;
				column = fSelected[currentItem];
				for (i = currentItem; i < count - 1; i++)
				{
					fSelected[i] = fSelected[i + 1];
					fSelected[i].fPosition = i;
				}
				fSelected[count - 1] = column;
				fSelected[count - 1].fPosition = count - 1;
				fListSelected.ResetContent();
				for (i = 0; i < (int) fSelected.size(); i++)
					fListSelected.AddString(fSelected[i].fTitle);
				fListSelected.SetCurrentSelection(count - 1);
				break;
			case IDC_BTN_UP:
				currentItem = fListSelected.GetCurrentSelection();
				if (currentItem == LB_ERR || currentItem == 0)
					break;
				column = fSelected[currentItem - 1];
				fSelected[currentItem - 1] = fSelected[currentItem];
				fSelected[currentItem] = column;
				fSelected[currentItem - 1].fPosition = currentItem - 1;
				fSelected[currentItem].fPosition = currentItem;
				fListSelected.ResetContent();
				for (i = 0; i < (int) fSelected.size(); i++)
					fListSelected.AddString(fSelected[i].fTitle);
				fListSelected.SetCurrentSelection(currentItem - 1);
				break;
			case IDC_BTN_DOWN:
				currentItem = fListSelected.GetCurrentSelection();
				if (currentItem == LB_ERR || currentItem == fListSelected.GetCount() - 1)
					break;
				column = fSelected[currentItem + 1];
				fSelected[currentItem + 1] = fSelected[currentItem];
				fSelected[currentItem] = column;
				fSelected[currentItem + 1].fPosition = currentItem + 1;
				fSelected[currentItem].fPosition = currentItem;
				fListSelected.ResetContent();
				for (i = 0; i < (int) fSelected.size(); i++)
					fListSelected.AddString(fSelected[i].fTitle);
				fListSelected.SetCurrentSelection(currentItem + 1);
				break;
			}
			break;
		}
		break;
	}
	return FALSE;
}


INT_PTR CALLBACK PreferencesListViewDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gDlgPreferencesListView->DialogProc(hDlg, message, wParam, lParam);
}
