/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "Game.h"

Game::Game(long game)
{
	string str;
	switch (game)
	{
	case kGameAll:
		fTitle = "All Games";
		fID = "ALL";
		fStdExe = "";
		fConnect = "";
		fGameDir = "";
		fPassword = "";
		fQueryCnt = 0;
		fStdPort = 0;
		break;
	case kGameQW:
		fTitle = "QuakeWorld";
		fID = "QW";
		fStdExe = "QW.EXE";
		fConnect = "+connect";
		fGameDir = "+gamedir";
		fPassword = "+password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("status");
		fStdPort = 27500;
		break;
	case kGameQ2:
		fTitle = "Quake 2";
		fID = "Q2";
		fStdExe = "QUAKE2.EXE";
		fConnect = "+connect";
		fGameDir = "+set game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("status");
		fStdPort = 27910;
		break;
	case kGameQ3:
		fTitle = "Quake 3 Arena";
		fID = "Q3A";
		fStdExe = "QUAKE3.EXE";
		fConnect = "+connect";
		fGameDir = "+set fs_game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 27960;
		break;
	case kGameHL:
		fTitle = "Half-Life";
		fID = "HL";
		fStdExe = "HL.EXE";
		fConnect = "+connect";
		fGameDir = "-game";
		fPassword = "+password";
		fQueryCnt = 3;
		fQueryStrings.push_back ("info");
		fQueryStrings.push_back ("players");
		fQueryStrings.push_back ("rules");
		fStdPort = 27015;
		break;
	case kGameUT:
		fTitle = "Unreal Tournament";
		fID = "UT";
		fStdExe = "UnrealTournament.EXE";
		fConnect = "";
		fGameDir = "+set game";
		fPassword = "";
		fQueryCnt = 1;
		fQueryStrings.push_back ("\\status\\");
		fStdPort = 7778;
		break;
	case kGameUT23:
		fTitle = "Unreal Tournament 2003";
		fID = "UT23";
		fStdExe = "UT2003.EXE";
		fConnect = "";
		fGameDir = "+set game";
		fPassword = "";
		fQueryCnt = 2;
		str.assign("\x078\x000\x000\x000\x000", 5);
		fQueryStrings.push_back(str);
		str.assign("\x078\x000\x000\x000\x003", 5);
		fQueryStrings.push_back(str);
		fStdPort = 10777;
		break;
	case kGameUT24:
		fTitle = "Unreal Tournament 2004";
		fID = "UT24";
		fStdExe = "UT2004.EXE";
		fConnect = "";
		fGameDir = "+set game";
		fPassword = "";
		fQueryCnt = 2;
		str.assign("\x07F\x000\x000\x000\x000", 5);
		fQueryStrings.push_back(str);
		str.assign("\x07F\x000\x000\x000\x003", 5);
		fQueryStrings.push_back(str);
		fStdPort = 10777;
		break;
	case kGameSOF:
		fTitle = "Soldier of Fortune";
		fID = "SOF";
		fStdExe = "SOF.EXE";
		fConnect = "+connect";
		fGameDir = "+set game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("status");
		fStdPort = 28910;
		break;
	case kGameEF:
		fTitle = "STV: Elite Force";
		fID = "EF";
		fStdExe = "STVOYHM.EXE";
		fConnect = "+connect";
		fGameDir = "+set fs_game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 27960;
		break;
	case kGameEF2:
		fTitle = "ST: Elite Force 2";
		fID = "EF2";
		fStdExe = "EF2.EXE";
		fConnect = "+set introcommand \"\" +connect";
		fGameDir = "+set fs_game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 29253;
		break;
	case kGameFK2:
		fTitle = "Heavy Metal FAKK2";
		fID = "FK2";
		fStdExe = "FAKK2.EXE";
		fConnect = "+connect";
		fGameDir = "+set fs_game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 27960;
		break;
	case kGameTBS:
		fTitle = "Tribes";
		fID = "TBS";
		fStdExe = "TRIBES.EXE";
		fConnect = "+connect";
		fGameDir = "-mod";
		fPassword = "+password";
		fQueryStrings.push_back ("\x060**");
		fQueryStrings.push_back ("b**");
		fQueryCnt = 1;
		fStdPort = 28001;
		break;
	case kGameTBS2:
		fTitle = "Tribes 2";
		fID = "TBS2";
		fStdExe = "TRIBES2.EXE";
		fConnect = "-nologin -connect";
		fGameDir = "-mod";
		fPassword = "-password";
		fQueryCnt = 1;
		str.assign("\x00E\x002\x001\x002\x003\x004\x000", 7);
		fQueryStrings.push_back (str);
		str.assign("\x012\x002\x001\x002\x003\x004\x000", 7);
		fQueryStrings.push_back (str);
		fStdPort = 28000;
		break;
	case kGameRTCW:
		fTitle = "Return to Castle Wolfenstein";
		fID = "RTCW";
		fStdExe = "WOLFMP.EXE";
		fConnect = "+connect";
		fGameDir = "+set fs_game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 27960;
		break;
	case kGameJK2:
		fTitle = "Jedi Knight 2: Jedi Outcast";
		fID = "JK2";
		fStdExe = "JK2MP.EXE";
		fConnect = "+connect";
		fGameDir = "+set fs_game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 28070;
		break;
	case kGameJK3:
		fTitle = "Jedi Knight 3: Jedi Academy";
		fID = "JK3";
		fStdExe = "JAMP.EXE";
		fConnect = "+connect";
		fGameDir = "+set fs_game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 29070;
		break;
	case kGameSOF2:
		fTitle = "Soldier of Fortune 2";
		fID = "SOF2";
		fStdExe = "SOF2MP.EXE";
		fConnect = "+connect";
		fGameDir = "+set fs_game";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 20100;
		break;
	case kGameBF42:
		fTitle = "Battlefield 1942";
		fID = "BF42";
		fStdExe = "BF1942.EXE";
		fConnect = "+restart 1 +joinServer";
		fGameDir = "+game";
		fPassword = "+password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("\\status\\");
		fStdPort = 22000;
		break;
	case kGameMOHA:
		fTitle = "Medal of Honor: Allied Assault";
		fID = "MOHA";
		fStdExe = "MOHAA.EXE";
		fConnect = "+connect";
		fGameDir = "";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("\x02getstatus");
		fStdPort = 12203;
		break;
	case kGameMOHS:
		fTitle = "Medal of Honor: Spearhead";
		fID = "MOHS";
		fStdExe = "MOH_SPEARHEAD.EXE";
		fConnect = "+connect";
		fGameDir = "";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("\x02getstatus");
		fStdPort = 12203;
		break;
	case kGameMOHB:
		fTitle = "Medal of Honor: Breakthrough";
		fID = "MOHB";
		fStdExe = "MOH_BREAKTHROUGH.EXE";
		fConnect = "+connect";
		fGameDir = "";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("\x02getstatus");
		fStdPort = 12203;
		break;
	case kGameCOD:
		fTitle = "Call of Duty";
		fID = "COD";
		fStdExe = "CoDMP.exe";
		fConnect = "+connect";
		fGameDir = "";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 28960;
		break;
	case kGameET:
		fTitle = "Wolfenstein - Enemy Territory";
		fID = "ET";
		fStdExe = "ET.exe";
		fConnect = "+connect";
		fGameDir = "";
		fPassword = "+set password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("getstatus");
		fStdPort = 27960;
		break;
	case kGameBFVN:
		fTitle = "Battlefield Vietnam";
		fID = "BFVN";
		fStdExe = "BFVIETNAM.EXE";
		fConnect = "+restart 1 +joinServer";
		fGameDir = "+game";
		fPassword = "+password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("\\status\\");
		fStdPort = 23000;
		break;
	case kGameFCRY:
		fTitle = "Far Cry";
		fID = "FCRY";
		fStdExe = "FARCRY.EXE";
		fConnect = "connect";
		fGameDir = "";
		fPassword = "cl_password";
		fQueryCnt = 1;
		fQueryStrings.push_back ("\x008\x080");
		fStdPort = 49001;
		break;
	}
}

Game::~Game()
{
}

vector<Game> gGames;
