/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "GameServer.h"
#include "Utilities.h"
#include "ServerList.h"

GameServer::GameServer()
:BaseServer()
{
	fFound = false;
	fListID = 0;
	fMissedCount = 0;
	fGame = 0;
	fPing = 0;
	fQueryPort = 0;
	fQueryPortStr = "";
	fHostName = "";
	fGameDir = "";
	fMapName = "";
	fDescription = "";
	fGameType = 0;
	fGameTypeStr = "";
	fDMFlags = 0;
	fMaxPlayers = 0;
	fCntPlayers = 0;
	fNeedPass = false;
	for (int i = 0; i < 5; i++)
		fGotPacket[i] = false;
}

GameServer::GameServer(unsigned long ip, unsigned short port)
:BaseServer(ip, port)
{
	fFound = false;
	fListID = 0;
	fMissedCount = 0;
	fGame = 0;
	fPing = 0;
	fQueryPort = port;
	fQueryPortStr = Port2String(port);
	fHostName = "";
	fGameDir = "";
	fMapName = "";
	fDescription = "";
	fGameType = 0;
	fGameTypeStr = "";
	fDMFlags = 0;
	fMaxPlayers = 0;
	fCntPlayers = 0;
	fNeedPass = false;
	for (int i = 0; i < 5; i++)
		fGotPacket[i] = false;
}

GameServer::GameServer(unsigned long ip, unsigned short port, unsigned short queryPort)
:BaseServer(ip, port)
{
	fFound = false;
	fListID = 0;
	fMissedCount = 0;
	fGame = 0;
	fPing = 0;
	fQueryPort = queryPort;
	fQueryPortStr = Port2String(queryPort);
	fHostName = "";
	fGameDir = "";
	fMapName = "";
	fDescription = "";
	fGameType = 0;
	fGameTypeStr = "";
	fDMFlags = 0;
	fMaxPlayers = 0;
	fCntPlayers = 0;
	fNeedPass = false;
	for (int i = 0; i < 5; i++)
		fGotPacket[i] = false;
}

void GameServer::SetAddress(unsigned long ip, unsigned short port)
{
	BaseServer::SetAddress(ip, port);
	this->fQueryPort = port;
	this->fQueryPortStr = Port2String(port);
}

void GameServer::SetAddress(unsigned long ip, unsigned short port, unsigned short queryPort)
{
	BaseServer::SetAddress(ip, port);
	this->fQueryPort = queryPort;
	this->fQueryPortStr = Port2String(queryPort);
}

void GameServer::SetVariable(StringVariable &variable)
{
	list<StringVariable>::iterator itVar;

	for (itVar = fVariables.begin(); itVar != fVariables.end(); itVar++)
	{
		if (itVar->fName == variable.fName)
		{
			itVar->fValue = variable.fValue;
			break;
		}
	}
	if (itVar == fVariables.end())
		fVariables.push_back(variable);
}

StringVariable *GameServer::GetVariable(string name)
{
	list<StringVariable>::iterator itVar;
	for (itVar = fVariables.begin(); itVar != fVariables.end(); itVar++)
	{
		if (itVar->fName == name)
			return itVar.operator ->();
	}
	return NULL;
}
