/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "GameTreeView.h"
#include "QFinder.h"
#include "Game.h"
#include "MainWindow.h"

GameTreeView::GameTreeView()
: TreeView()
{
}

GameTreeView::~GameTreeView()
{
}

void GameTreeView::Create(HWND hParent)
{
	long index;
	long game;

	TreeView::Create(hParent, IDC_GAME_TREEVIEW);
	SetStyleFullRowSelect();
	fImageList = ImageList_LoadImage(gQFinder->fInstance, MAKEINTRESOURCE (IDB_GAMEICONS_BIG_16M), 24, 0, CLR_NONE, IMAGE_BITMAP, LR_CREATEDIBSECTION);
	SetImageList(fImageList);
	TreeView_SetIndent(fHandle, 0);

	gGames[kGameAll].fTreeItem = AddItem(TVI_ROOT, gGames[kGameAll].fTitle, 0, 0);
	for (index = 1; index <= kGameCount; index++)
	{
		game = gQFinder->fSettings.fGame.fGameSort[index];
		gGames[game].fTreeItem = AddItem(gGames[kGameAll].fTreeItem, gGames[game].fTitle, game, game);
	}
	Expand(gGames[kGameAll].fTreeItem, TVE_EXPAND);
}

void GameTreeView::Rebuild()
{
	HWND hParent = fParent;

	DestroyWindow(fHandle);
	Create(hParent);
}

LRESULT GameTreeView::OnNotify(LPARAM lParam)
{
	NMTREEVIEW *nmTreeView = (NMTREEVIEW *) lParam;
	long game;
	MainToolBar *toolbar;
	HMENU menu;

	switch (nmTreeView->hdr.code)
	{
	case TVN_ITEMEXPANDING:
		if (nmTreeView->action != TVE_EXPAND)
		{
			return TRUE;
		}
		break;
	case TVN_SELCHANGED:
		game = nmTreeView->itemNew.lParam;
		gMainWindow->SetCurrentGame(game);
		toolbar = gMainWindow->GetMainToolBar();
		toolbar->EnableButton(ID_ACTIONS_JOINGAME, false);
		toolbar->EnableButton(ID_ACTIONS_UPDATESINGLESERVER, false);
		menu = GetMenu(fParent);
		EnableMenuItem(menu, ID_ACTIONS_JOINGAME, MF_GRAYED);
		EnableMenuItem(menu, ID_ACTIONS_UPDATESINGLESERVER, MF_GRAYED);
		if (game == kGameAll)
		{
			toolbar->ChangeBitmap(ID_ACTIONS_UPDATESELECTEDGAME, kToolBarIndexRefreshGame);
			toolbar->EnableButton(ID_ACTIONS_UPDATESELECTEDGAME, false);
			toolbar->Move(0, 0, 0, 0);
			EnableMenuItem(menu, ID_ACTIONS_UPDATESELECTEDGAME, MF_GRAYED);
		}
		else
		{
			toolbar->ChangeBitmap(ID_ACTIONS_UPDATESELECTEDGAME, kToolBarIndexRefreshGame + game);
			toolbar->EnableButton(ID_ACTIONS_UPDATESELECTEDGAME);
			toolbar->Move(0, 0, 0, 0);
			EnableMenuItem(menu, ID_ACTIONS_UPDATESELECTEDGAME, MF_ENABLED);
		}
		gMainWindow->MarkOldServer();
		break;
	}
	return 0;
}

void GameTreeView::SelectLastGame()
{
	long index;

	for (index = 0; index <= kGameCount; index++)
		if (gQFinder->fSettings.fGame.fLastSelected == gGames[index].fID)
			break;
	if (index == kGameCount + 1)
		index = kGameAll;
	SelectItem(gGames[index].fTreeItem);
}
