/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "HalflifeRefresh.h"
#include "SocketWindow.h"
#include "Game.h"

DWORD WINAPI HalflifeRefreshThread(LPVOID param);
CRITICAL_SECTION csStop;
CRITICAL_SECTION csQueue;

const unsigned long kTimeout = 1000;
const unsigned long kMaxTimeout = 3;
const unsigned long kMaxTry = 5;

bool HalflifeRefresh::fRunning = false;
bool HalflifeRefresh::fStop = false;
HANDLE HalflifeRefresh::fHandle = 0;
HWND HalflifeRefresh::fWindow = 0;
unsigned long HalflifeRefresh::fMessage = 0;
list<RefreshServer> HalflifeRefresh::fServers;

HalflifeRefresh::HalflifeRefresh()
{
	InitializeCriticalSection(&csStop);
	InitializeCriticalSection(&csQueue);
}

HalflifeRefresh::~HalflifeRefresh()
{
	DeleteCriticalSection(&csStop);
	DeleteCriticalSection(&csQueue);
}

void HalflifeRefresh::Start(HWND hwnd, unsigned int message)
{
	if (!fRunning)
	{
		fRunning = true;
		fStop = false;
		fWindow = hwnd;
		fMessage = message;
		CreateThread(NULL, 0, HalflifeRefreshThread, NULL, 0, (LPDWORD) &fHandle);
	}
}

void HalflifeRefresh::Stop()
{
	if (fRunning)
	{
		EnterCriticalSection(&csStop);
		fStop = true;
		LeaveCriticalSection(&csStop);
		fRunning = false;
	}
}

void HalflifeRefresh::AddServer(GameServer *server, SOCKET sock)
{
	list<RefreshServer>::iterator itServer;
	RefreshServer *refresh;

	string message;
	bool found = false;
	message = "Add " + server->fIPStr + ":" + server->fPortStr;
	gLogFile->DebugMessage(message);
	refresh = new RefreshServer(server);
	refresh->fNew = true;
	refresh->fSocket = sock;
	refresh->fLastTime = GetTickCount();
	refresh->fTimeoutCount = 0;
	refresh->fTryCount = 0;
	EnterCriticalSection(&csQueue);
	for (itServer = fServers.begin(); itServer != fServers.end(); itServer++)
	{
		if (itServer->fGame == refresh->fGame &&
			itServer->fIP == refresh->fIP &&
			itServer->fPort == refresh->fPort)
		{
//			fServers.erase(itServer);
			found = true;
			for (int i = 0; i < 5; i++)
				itServer->fGotPacket[i] = refresh->fGotPacket[i];
			break;
		}
	}
	if (!found)
		fServers.push_back(*refresh);
	LeaveCriticalSection(&csQueue);

	delete refresh;
}

void HalflifeRefresh::PutServer(RefreshServer &server)
{
	list<RefreshServer>::iterator itServer;

	string message;
	message = "Put " + server.fIPStr + ":" + server.fPortStr;
	gLogFile->DebugMessage(message);
	EnterCriticalSection(&csQueue);
	for (itServer = fServers.begin(); itServer != fServers.end(); itServer++)
	{
		if (itServer->fGame == server.fGame &&
			itServer->fIP == server.fIP &&
			itServer->fPort == server.fPort)
		{
			break;
		}
	}
	if (itServer == fServers.end())
		fServers.push_back(server);
	LeaveCriticalSection(&csQueue);
}

DWORD WINAPI HalflifeRefreshThread(LPVOID param)
{
	bool stop;
	int err;
	struct sockaddr_in toAddr;
	string str;
	int size;
	int index;
	int packet;
	bool complete;
	unsigned long time;

	toAddr.sin_family = AF_INET;

	while (true)
	{
		EnterCriticalSection(&csStop);
		stop = HalflifeRefresh::fStop;
		LeaveCriticalSection(&csStop);
		if (stop)
			break;

		EnterCriticalSection(&csQueue);
		size = HalflifeRefresh::fServers.size();
		LeaveCriticalSection(&csQueue);

		for (index = 0; index < size; index++)
		{
			EnterCriticalSection(&csQueue);
			RefreshServer server = HalflifeRefresh::fServers.front();
			HalflifeRefresh::fServers.pop_front();
			LeaveCriticalSection(&csQueue);

			toAddr.sin_addr.S_un.S_addr = htonl(server.fIP);
			toAddr.sin_port = htons(server.fQueryPort);

			time = GetTickCount();
			complete = true;
			if (server.fNew ||
				((time - server.fLastTime > kTimeout) &&
				 (server.fTimeoutCount < kMaxTimeout)))
			{
				if (!server.fNew)
				{
					server.fTimeoutCount++;
				}
				for (packet = 0; packet < server.fPacketCount; packet++)
				{
					if (server.fGotPacket[packet] == false)
					{
						complete = false;
						str = gGames[server.fGame].fQueryStrings[packet];
						string message;
						message = "Send " + server.fIPStr + ":" + server.fPortStr + ": " + str;
						gLogFile->DebugMessage(message);
						if (sendto(server.fSocket, str.c_str(), str.length(), 0, (sockaddr *) &toAddr, sizeof(sockaddr)) == SOCKET_ERROR)
						{
							err = WSAGetLastError();
						}
					}
				}
				server.fLastTime = GetTickCount();

				if (!complete)
				{
					server.fNew = false;
					HalflifeRefresh::PutServer(server);
				}
			}
			else
			{
				server.fTryCount++;
				if (server.fTimeoutCount < kMaxTimeout && server.fTryCount < kMaxTry)
					HalflifeRefresh::PutServer(server);
			}
		}
		Sleep(200);
	}
	ExitThread(0);
}

HalflifeRefresh *gHalflifeRefresh;