/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "MainWindow.h"
#include "QFinder.h"
#include "Globals.h"
#include "DlgPreferences.h"
#include "DlgNetwork.h"
#include "DlgAbout.h"
#include "DlgPassword.h"
#include "Game.h"
#include "ServerStatus.h"
#include "Parser.h"
#include "HalflifeRefresh.h"

MainWindow::MainWindow()
{
	WNDCLASS wndClass;
	
	wndClass.style         = CS_HREDRAW | CS_VREDRAW;
	wndClass.lpfnWndProc   = MainWndProc;
	wndClass.cbClsExtra    = 0;
	wndClass.cbWndExtra    = 0;
	wndClass.hInstance     = gQFinder->fInstance;
	wndClass.hIcon         = gQFinder->fIcon;
	wndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wndClass.hbrBackground = (HBRUSH) (COLOR_ACTIVEBORDER + 1);
	wndClass.lpszMenuName  = MAKEINTRESOURCE(IDR_MAINMENU);
	wndClass.lpszClassName = gQFinder->fAppName.c_str();
	
	if (!RegisterClass(&wndClass))
	{
		return;
	}
	fCurrentServer = NULL;
}

MainWindow::~MainWindow()
{
}

void MainWindow::Create()
{
	fLayout = gQFinder->fSettings.fMainWindow.fLayout;
	fLayoutParameter1 = gQFinder->fSettings.fMainWindow.fLayoutParameter1;
	fLayoutParameter2 = gQFinder->fSettings.fMainWindow.fLayoutParameter2;
	fTVWidth = gQFinder->fSettings.fMainWindow.fTVWidth;
	fLVHeight = gQFinder->fSettings.fMainWindow.fLVHeight;
	fLVWidth = gQFinder->fSettings.fMainWindow.fLVWidth;
	fWnd = CreateWindow(gQFinder->fAppName.c_str(),
						gQFinder->fAppName.c_str(),
						WS_OVERLAPPEDWINDOW,
						gQFinder->fSettings.fMainWindow.fLeft,
						gQFinder->fSettings.fMainWindow.fTop,
						gQFinder->fSettings.fMainWindow.fWidth,
						gQFinder->fSettings.fMainWindow.fHeight,
						NULL,
						NULL,
						gQFinder->fInstance,
						NULL);
	fDragMode = kDragNone;
	fOldWidth = gQFinder->fSettings.fMainWindow.fWidth;
	fOldHeight = gQFinder->fSettings.fMainWindow.fHeight;

	gSocketWindow = new SocketWindow(this);
	gHalflifeRefresh->Start(fWnd, kMsgNetwork);
}

void MainWindow::Show()
{
	if (gQFinder->fSettings.fMainWindow.fMode == kWindowModeMinimized)
	{
		ShowWindow(fWnd, SW_MINIMIZE);
	}
	else if (gQFinder->fSettings.fMainWindow.fMode == kWindowModeMaximized)
	{
		ShowWindow(fWnd, SW_MAXIMIZE);
	}
	else
	{
		ShowWindow(fWnd, SW_SHOW);
	}
	UpdateAllGames();
	if (gQFinder->fSettings.fHotkeys.minimizeActive)
		RegisterHotKey(fWnd,
			gQFinder->fSettings.fHotkeys.minimizeId,
			gQFinder->fSettings.fHotkeys.minimizeModifier,
			gQFinder->fSettings.fHotkeys.minimizeVirtualKey);
}

int MainWindow::SendMsg(UINT message, WPARAM wParam, LPARAM lParam)
{
	return SendMessage(fWnd, message, wParam, lParam);
}

LRESULT MainWindow::WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_COMMAND:
		OnCommand(wParam, lParam);
		break;
	case WM_CREATE:
		OnCreate(hWnd);
		break;
	case WM_DESTROY:
		if (gQFinder->fSettings.fMainWindow.fMode == kWindowModeNormal)
		{
			RECT rect;
			GetWindowRect(fWnd, &rect);
			gQFinder->fSettings.fMainWindow.fLeft = rect.left;
			gQFinder->fSettings.fMainWindow.fTop = rect.top;
			gQFinder->fSettings.fMainWindow.fWidth = rect.right - rect.left;
			gQFinder->fSettings.fMainWindow.fHeight = rect.bottom - rect.top;
		}
		if (gQFinder->fSettings.fMainWindow.fMode == kWindowModeMaximized)
		{
			RECT rect;
			int width;
			int border;
			int newWidth;

			fLVHeight = (fLVHeight * (double) gQFinder->fSettings.fMainWindow.fHeight) / (double) fOldHeight;

			GetWindowRect(fWnd, &rect);
			width = rect.right - rect.left;
			GetClientRect(fWnd, &rect);
			border = width - (rect.right - rect.left);
			fServerListView.GetRect(rect);
			width = rect.right - rect.left + 1;
			newWidth = gQFinder->fSettings.fMainWindow.fWidth - border - (long) fTVWidth - 3 + 1;
			fLVWidth = (fLVWidth * (double) newWidth) / (double) width;
		}
		for (int i = 0; i < (int) gQFinder->fSettings.fMainWindow.fServerColumns.size(); i++)
		{
			gQFinder->fSettings.fMainWindow.fServerColumns[i].fWidth = fServerListView.GetColumnWidth(gQFinder->fSettings.fMainWindow.fServerColumns[i].fPosition);
		}
		for (int i = 0; i < (int) gQFinder->fSettings.fMainWindow.fPlayerColumns.size(); i++)
		{
			gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fWidth = fPlayerListView.GetColumnWidth(gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fPosition);
		}
		for (int i = 0; i < (int) gQFinder->fSettings.fMainWindow.fVariableColumns.size(); i++)
		{
			gQFinder->fSettings.fMainWindow.fVariableColumns[i].fWidth = fVariableListView.GetColumnWidth(gQFinder->fSettings.fMainWindow.fVariableColumns[i].fPosition);
		}
		if (gQFinder->fSettings.fMainWindow.fMode != kWindowModeMinimized)
		{
			gQFinder->fSettings.fMainWindow.fTVWidth = (long) fTVWidth;
			gQFinder->fSettings.fMainWindow.fLVHeight = (long) fLVHeight;
			gQFinder->fSettings.fMainWindow.fLVWidth = (long) fLVWidth;
		}
		else
		{
			if (gQFinder->fSettings.fMainWindow.fMinimizeMode == kMinimizeModeTray)
			{
				NOTIFYICONDATA	nid;
				nid.cbSize = sizeof(NOTIFYICONDATA);
				nid.hWnd = fWnd;
				nid.uID = IDC_TRAYICON;

				Shell_NotifyIcon(NIM_DELETE, &nid);
			}
		}
		gQFinder->fSettings.fGame.fLastSelected = gGames[fCurrentGame].fID;
		if (gQFinder->fSettings.fHotkeys.minimizeActive)
			UnregisterHotKey(fWnd, gQFinder->fSettings.fHotkeys.minimizeId);
		PostQuitMessage(0);
		return 0;
	case WM_LBUTTONDOWN:
		OnLeftButtonDown(wParam, lParam);
		break;
	case WM_LBUTTONUP:
		OnLeftButtonUp(wParam, lParam);
		break;
	case WM_MOUSEMOVE:
		OnMouseMove(wParam, lParam);
		break;
	case WM_HOTKEY:
		if (gQFinder->fSettings.fMainWindow.fMode != kWindowModeMinimized)
		{
			WINDOWINFO wInfo;
			wInfo.cbSize = sizeof(WINDOWINFO);
			wInfo.dwWindowStatus = 0;
			GetWindowInfo(fWnd, &wInfo);
			if (wInfo.dwWindowStatus == WS_ACTIVECAPTION)
			{
				ShowWindow(fWnd, SW_MINIMIZE);
			}
			else
			{
				ShowWindow(fWnd, SW_MINIMIZE);
				if (gQFinder->fSettings.fMainWindow.fMinimizeMode == kMinimizeModeTray)
					SendMsg(kMsgTrayIcon, 0, WM_LBUTTONDBLCLK);
				else
					ShowWindow(fWnd, SW_RESTORE);
			}
		}
		else
		{
			if (gQFinder->fSettings.fMainWindow.fMinimizeMode == kMinimizeModeTray)
				SendMsg(kMsgTrayIcon, 0, WM_LBUTTONDBLCLK);
			else
				ShowWindow(fWnd, SW_RESTORE);
			if (gQFinder->fSettings.fHotkeys.minimizeRefresh)
				UpdateAllGames();
		}
		break;
	case WM_NOTIFY:
		switch (LOWORD(wParam))
		{
		case IDC_GAME_TREEVIEW:
			return fGameTreeView.OnNotify(lParam);
		case IDC_SERVER_LISTVIEW:
			return fServerListView.OnNotify(lParam);
		case IDC_PLAYER_LISTVIEW:
			return fPlayerListView.OnNotify(lParam);
		case IDC_VARIABLE_LISTVIEW:
			return fVariableListView.OnNotify(lParam);
		}
		break;
	case WM_SIZING:
		return OnSizing(wParam, lParam);
		break;
	case WM_SIZE:
		return OnSize(wParam, lParam);
		break;
	case kMsgTrayIcon:
		{
			NOTIFYICONDATA	nid;
			HMENU	menu;
			POINT	pt;

			switch (lParam)
			{
			case WM_LBUTTONDBLCLK:
				nid.cbSize = sizeof(NOTIFYICONDATA);
				nid.hWnd = fWnd;
				nid.uID = IDC_TRAYICON;

				Shell_NotifyIcon(NIM_DELETE, &nid);
				ShowWindow(fWnd, SW_SHOW);
				ShowWindow(fWnd, SW_RESTORE);
				break;
			case WM_RBUTTONUP:
				menu = LoadMenu(gQFinder->fInstance, MAKEINTRESOURCE (IDR_TRAYMENU));
				menu = GetSubMenu(menu, 0);
				GetCursorPos(&pt);
				SetForegroundWindow(fWnd);
				TrackPopupMenu(menu, 0, pt.x, pt.y, 0, fWnd, NULL);
				break;
			case WM_LBUTTONUP:
				break;
			}
		return 0;
		}
	case kMsgNetwork:
		OnNetwork(wParam, lParam);
		break;
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}

MainToolBar *MainWindow::GetMainToolBar()
{
	return &fMainToolBar;
}

ServerListView *MainWindow::GetServerListView()
{
	return &fServerListView;
}

PlayerListView *MainWindow::GetPlayerListView()
{
	return &fPlayerListView;
}

VariableListView *MainWindow::GetVariableListView()
{
	return &fVariableListView;
}

void MainWindow::SetCurrentGame(long game)
{
	ServerList *svrList;
	set<GameServer>::iterator itServer;

	fCurrentGame = game;
	fServerListView.DeleteAllItems();
	fPlayerListView.DeleteAllItems();
	fVariableListView.DeleteAllItems();
	svrList = gQFinder->GetServerList(gQFinder->fBroadcastID);
	for (itServer = svrList->fServers.begin();
		 itServer != svrList->fServers.end();
		 itServer++)
	{
		AddServer(itServer.operator ->());
	}
	svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
	for (itServer = svrList->fServers.begin();
		 itServer != svrList->fServers.end();
		 itServer++)
	{
		AddServer(itServer.operator ->());
	}
}

long MainWindow::GetCurrentGame()
{
	return fCurrentGame;
}

void MainWindow::SetCurrentServer(GameServer *server)
{
	fCurrentServer = server;
	if (server != NULL)
	{
		fMainToolBar.EnableButton(ID_ACTIONS_JOINGAME, true);
		fMainToolBar.EnableButton(ID_ACTIONS_UPDATESINGLESERVER, true);
		HMENU menu = GetMenu(fWnd);
		EnableMenuItem(menu, ID_ACTIONS_JOINGAME, MF_ENABLED);
		EnableMenuItem(menu, ID_ACTIONS_UPDATESINGLESERVER, MF_ENABLED);
	}
	else
	{
		fMainToolBar.EnableButton(ID_ACTIONS_JOINGAME, false);
		fMainToolBar.EnableButton(ID_ACTIONS_UPDATESINGLESERVER, false);
		HMENU menu = GetMenu(fWnd);
		EnableMenuItem(menu, ID_ACTIONS_JOINGAME, MF_GRAYED);
		EnableMenuItem(menu, ID_ACTIONS_UPDATESINGLESERVER, MF_GRAYED);
	}
}

GameServer *MainWindow::GetCurrentServer()
{
	return fCurrentServer;
}

void MainWindow::SetOldServer(GameServer *server)
{
	if (server)
	{
		fOldServer.fGame = server->fGame;
		fOldServer.fIP = server->fIP;
		fOldServer.fPort = server->fPort;
		fOldServer.fQueryPort = server->fQueryPort;
	}
}

LRESULT MainWindow::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam))
	{
	case ID_ACTIONS_JOINGAME:
		JoinGame();
		break;
	case ID_ACTIONS_UPDATEALLGAMES:
		UpdateAllGames();
		break;
	case ID_ACTIONS_UPDATESELECTEDGAME:
		UpdateSelectedGame();
		break;
	case ID_ACTIONS_UPDATESINGLESERVER:
		UpdateSingleServer();
		break;
	case ID_ACTIONS_QUIT:
		SendMessage (fWnd, WM_DESTROY, 0, 0);
		break;
	case ID_OPTIONS_PREFERENCES:
		Preferences();
		break;
	case ID_OPTIONS_NETWORK:
		{
			DlgNetwork *dialog = new DlgNetwork(fWnd);
			dialog->Create();
			delete dialog;
			break;
		}
	case ID_HELP_ABOUT:
		{
			DlgAbout *dialog = new DlgAbout(fWnd);
			dialog->Create();
			delete dialog;
			break;
		}
	}
	return 0;
}

LRESULT MainWindow::OnCreate(HWND hWnd)
{
	RECT rect;
	long top;
	long left;

	fMainToolBar.Create(hWnd);
	fGameTreeView.Create(hWnd);
	fServerListView.Create(hWnd);
	fPlayerListView.Create(hWnd);
	fVariableListView.Create(hWnd);

	fMainToolBar.Move(0, 0, 0, 0);
	fMainToolBar.GetRect(rect);
	top = rect.bottom - rect.top + 1;
	GetClientRect (hWnd, &rect);
	switch (fLayout)
	{
	case kLayout_Standard:
		if (fLayoutParameter1 == kLayout_TreeViewShort)
		{
			fGameTreeView.Move(0, top, (long) fTVWidth, (long) fLVHeight);
			left = (long) fTVWidth + 3;
			fServerListView.Move(left, top, rect.right - left, (long) fLVHeight);
			left = 0;
		}
		if (fLayoutParameter1 == kLayout_TreeViewLong)
		{
			fGameTreeView.Move(0, top, (long) fTVWidth, rect.bottom - top);
			left = (long) fTVWidth + 3;
			fServerListView.Move(left, top, rect.right - left, (long) fLVHeight);
		}
		top += (long) fLVHeight + 3;
		if (fLayoutParameter2 == kLayout_PlayerLeft)
		{
			fPlayerListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
			left += (long) fLVWidth + 3;
			fVariableListView.Move(left, top, rect.right - left, rect.bottom - top);
		}
		if (fLayoutParameter2 == kLayout_PlayerRight)
		{
			fVariableListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
			left += (long) fLVWidth + 3;
			fPlayerListView.Move(left, top, rect.right - left, rect.bottom - top);
		}
		break;
	}
	fMainToolBar.Show();
	fGameTreeView.Show();
	fServerListView.Show();
	fPlayerListView.Show();
	fVariableListView.Show();

	fGameTreeView.SelectLastGame();

	return 0;
}

LRESULT MainWindow::OnLeftButtonDown(WPARAM wParam, LPARAM lParam)
{
	POINTS point;
	RECT rect;
	long top;
	long left;

	point = MAKEPOINTS(lParam);
	fMainToolBar.GetRect(rect);
	top = rect.bottom - rect.top + 1;
	GetClientRect(fWnd, &rect);
	switch (fLayout)
	{
	case kLayout_Standard:
		if (fLayoutParameter1 == kLayout_TreeViewShort)
		{
			if (point.x >= (long) fTVWidth && point.x <= (long) fTVWidth + 2 &&
				point.y >= top && point.y <= top + (long) fLVHeight - 1)
			{
				fDragMode = kDragTVWidth;
				fDragStart = point.x;
				fDragLast = fDragStart;
				fDragDelta = fDragStart - (long) fTVWidth;
				rect.left = (long) fTVWidth - 1;
				rect.top = top;
				rect.right = rect.left + 5;
				rect.bottom = top + (long) fLVHeight;
				DrawXORRect(rect);
				SetCapture(fWnd);
				SetCursor(LoadCursor(NULL, IDC_SIZEWE));
			}
			top += (long) fLVHeight;
			if (point.x >= 0 && point.x <= rect.right &&
				point.y >= top && point.y <= top + 2)
			{
				fDragMode = kDragLVHeight;
				fDragStart = point.y;
				fDragLast = fDragStart;
				fDragDelta = fDragStart - top;
				rect.top = top - 1;
				rect.bottom = rect.top + 5;
				DrawXORRect(rect);
				SetCapture(fWnd);
				SetCursor(LoadCursor(NULL, IDC_SIZENS));
			}
			top += 3;
			if (point.x >= (long) fLVWidth && point.x <= (long) fLVWidth + 2 &&
				point.y >= top && point.y <= rect.bottom)
			{
				fDragMode = kDragLVWidth;
				fDragStart = point.x;
				fDragLast = fDragStart;
				fDragDelta = fDragStart - (long) fLVWidth;
				rect.left = (long) fLVWidth - 1;
				rect.top = top;
				rect.right = rect.left + 5;
				DrawXORRect(rect);
				SetCapture(fWnd);
				SetCursor(LoadCursor(NULL, IDC_SIZEWE));
			}
		}
		if (fLayoutParameter1 == kLayout_TreeViewLong)
		{
			if (point.x >= (long) fTVWidth && point.x <= (long) fTVWidth + 2 &&
				point.y >= top && point.y <= rect.bottom)
			{
				fDragMode = kDragTVWidth;
				fDragStart = point.x;
				fDragLast = fDragStart;
				fDragDelta = fDragStart - (long) fTVWidth;
				rect.left = (long) fTVWidth - 1;
				rect.top = top;
				rect.right = rect.left + 5;
				DrawXORRect(rect);
				SetCapture(fWnd);
				SetCursor(LoadCursor(NULL, IDC_SIZEWE));
			}
			top += (long) fLVHeight;
			left = (long) fTVWidth + 3;
			if (point.x >= left && point.x <= rect.right &&
				point.y >= top && point.y <= top + 2)
			{
				fDragMode = kDragLVHeight;
				fDragStart = point.y;
				fDragLast = fDragStart;
				fDragDelta = fDragStart - top;
				rect.left = left;
				rect.top = top - 1;
				rect.bottom = rect.top + 5;
				DrawXORRect(rect);
				SetCapture(fWnd);
				SetCursor(LoadCursor(NULL, IDC_SIZENS));
			}
			left += (long) fLVWidth;
			top += 3;
			if (point.x >= left && point.x <= left + 2 &&
				point.y >= top && point.y <= rect.bottom)
			{
				fDragMode = kDragLVWidth;
				fDragStart = point.x;
				fDragLast = fDragStart;
				fDragDelta = fDragStart - left;
				rect.left = left - 1;
				rect.top = top;
				rect.right = rect.left + 5;
				DrawXORRect(rect);
				SetCapture(fWnd);
				SetCursor(LoadCursor(NULL, IDC_SIZEWE));
			}
		}
		break;
	default:
		break;
	}

	return 0;
}

LRESULT MainWindow::OnLeftButtonUp(WPARAM wParam, LPARAM lParam)
{
	POINTS point;
	RECT rect;
	long top;

	point = MAKEPOINTS(lParam);
	fMainToolBar.GetRect(rect);
	top = rect.bottom - rect.top + 1;
	GetClientRect(fWnd, &rect);
	switch (fDragMode)
	{
	case kDragTVWidth:
		rect.top = top;
		rect.left = fDragLast - fDragDelta - 1;
		rect.right = rect.left + 5;
		if (fLayoutParameter1 == kLayout_TreeViewShort)
			rect.bottom = rect.top + (long) fLVHeight;
		DrawXORRect(rect);
		ReleaseCapture();
		fDragMode = kDragNone;
		fTVWidth = fDragLast - fDragDelta;
		ResizeControls(kDragTVWidth, true);
		return 0;
	case kDragLVHeight:
		rect.top = fDragLast - fDragDelta - 1;
		rect.bottom = rect.top + 5;
		if (fLayoutParameter1 == kLayout_TreeViewLong)
			rect.left = (long) fTVWidth + 3;
		DrawXORRect(rect);
		ReleaseCapture();
		fDragMode = kDragNone;
		fMainToolBar.GetRect(rect);
		fLVHeight = fDragLast - fDragDelta - (rect.bottom - rect.top + 1);
		ResizeControls(kDragLVHeight, true);
		return 0;
	case kDragLVWidth:
		rect.top = top + (long) fLVHeight + 3;
		rect.left = fDragLast - fDragDelta - 1;
		rect.right = rect.left + 5;
		DrawXORRect(rect);
		ReleaseCapture();
		fDragMode = kDragNone;
		fLVWidth = fDragLast - fDragDelta;
		if (fLayoutParameter1 == kLayout_TreeViewLong)
			fLVWidth -= (long) fTVWidth + 3;
		ResizeControls(kDragLVWidth, true);
		return 0;
	}
	return 0;
}

LRESULT MainWindow::OnMouseMove (WPARAM wParam, LPARAM lParam)
{
	POINTS point;
	RECT rect;
	RECT clientRect;
	long top;
	long left;

	point = MAKEPOINTS(lParam);
	fMainToolBar.GetRect(rect);
	top = rect.bottom - rect.top + 1;
	GetClientRect(fWnd, &rect);
	switch (fDragMode)
	{
	case kDragTVWidth:
		rect.top = top;
		rect.left = fDragLast - fDragDelta - 1;
		rect.right = rect.left + 5;
		if (fLayoutParameter1 == kLayout_TreeViewShort)
			rect.bottom = rect.top + (long) fLVHeight;
		DrawXORRect(rect);
		fDragLast = point.x;
		GetClientRect(fWnd, &clientRect);
		if (fDragLast < 10)
			fDragLast = 10;
		if (fDragLast > 250)
			fDragLast = 250;
		rect.left = fDragLast - fDragDelta - 1;
		rect.right = rect.left + 5;
		DrawXORRect(rect);
		SetCursor(LoadCursor(NULL, IDC_SIZEWE));
		return 0;
	case kDragLVHeight:
		rect.top = fDragLast - fDragDelta - 1;
		rect.bottom = rect.top + 5;
		if (fLayoutParameter1 == kLayout_TreeViewLong)
			rect.left = (long) fTVWidth + 3;
		DrawXORRect(rect);
		fDragLast = point.y;
		GetClientRect(fWnd, &clientRect);
		if (fDragLast < top + 10)
			fDragLast = top + 10;
		if (fDragLast > clientRect.bottom - 10)
			fDragLast = clientRect.bottom - 10;
		rect.top = fDragLast - fDragDelta - 1;
		rect.bottom = rect.top + 5;
		DrawXORRect(rect);
		SetCursor(LoadCursor(NULL, IDC_SIZENS));
		return 0;
	case kDragLVWidth:
		rect.top = top + (long) fLVHeight + 3;
		rect.left = fDragLast - fDragDelta - 1;
		rect.right = rect.left + 5;
		DrawXORRect(rect);
		fDragLast = point.x;
		GetClientRect(fWnd, &clientRect);
		if (fLayoutParameter1 == kLayout_TreeViewLong)
			clientRect.left = (long) fTVWidth + 3;
		if (fDragLast < clientRect.left + 10)
			fDragLast = clientRect.left + 10;
		if (fDragLast > clientRect.right - 10)
			fDragLast = clientRect.right - 10;
		rect.left = fDragLast - fDragDelta - 1;
		rect.right = rect.left + 5;
		DrawXORRect(rect);
		SetCursor(LoadCursor(NULL, IDC_SIZEWE));
		return 0;
	}
	fMainToolBar.GetRect (rect);
	top = rect.bottom - rect.top + 1;
	GetClientRect(fWnd, &rect);
	switch (fLayout)
	{
	case kLayout_Standard:
		if (fLayoutParameter1 == kLayout_TreeViewShort)
		{
			if (point.x >= (long) fTVWidth && point.x <= (long) fTVWidth + 2 &&
				point.y >= top && point.y <= top + (long) fLVHeight - 1)
			{
				SetCursor(LoadCursor(NULL, IDC_SIZEWE));
			}
			top += (long) fLVHeight;
			if (point.x >= 0 && point.x <= rect.right &&
				point.y >= top && point.y <= top + 2)
			{
				SetCursor(LoadCursor(NULL, IDC_SIZENS));
			}
			top += 3;
			if (point.x >= (long) fLVWidth && point.x <= (long) fLVWidth + 2 &&
				point.y >= top && point.y <= rect.bottom)
			{
				SetCursor(LoadCursor(NULL, IDC_SIZEWE));
			}
		}
		if (fLayoutParameter1 == kLayout_TreeViewLong)
		{
			if (point.x >= (long) fTVWidth && point.x <= (long) fTVWidth + 2 &&
				point.y >= top && point.y <= rect.bottom)
			{
				SetCursor(LoadCursor(NULL, IDC_SIZEWE));
			}
			top += (long) fLVHeight;
			left = (long) fTVWidth + 3;
			if (point.x >= left && point.x <= rect.right &&
				point.y >= top && point.y <= top + 2)
			{
				SetCursor(LoadCursor(NULL, IDC_SIZENS));
			}
			left += (long) fLVWidth;
			top += 3;
			if (point.x >= left && point.x <= left + 2 &&
				point.y >= top && point.y <= rect.bottom)
			{
				SetCursor(LoadCursor(NULL, IDC_SIZEWE));
			}
		}
		break;
	default:
		break;
	}
	return 0;
}

LRESULT MainWindow::OnSizing(WPARAM wParam, LPARAM lParam)
{
	RECT *rect = (RECT *) lParam;
	if ((rect->right - rect->left) < kMinWidth)
	{
		if (wParam == WMSZ_LEFT ||
			wParam == WMSZ_TOPLEFT ||
			wParam == WMSZ_BOTTOMLEFT)
		{
			rect->left = rect->right - kMinWidth;
		}
		else
		{
			rect->right = rect->left + kMinWidth;
		}
	}
	if ((rect->bottom - rect->top) < kMinHeight)
	{
		if (wParam == WMSZ_TOP ||
			wParam == WMSZ_TOPLEFT ||
			wParam == WMSZ_TOPRIGHT)
		{
			rect->top = rect->bottom - kMinHeight;
		}
		else
		{
			rect->bottom = rect->top + kMinHeight;
		}
	}
	return TRUE;
}

LRESULT MainWindow::OnSize(WPARAM wParam, LPARAM lParam)
{
	RECT rect;
	switch (wParam)
	{
	case SIZE_MAXIMIZED:
		gQFinder->fSettings.fMainWindow.fMode = kWindowModeMaximized;
		break;
	case SIZE_MINIMIZED:
		gQFinder->fSettings.fMainWindow.fMode = kWindowModeMinimized;
		break;
	case SIZE_RESTORED:
		gQFinder->fSettings.fMainWindow.fMode = kWindowModeNormal;
		break;
	}
	if (wParam == SIZE_MINIMIZED)
	{
		if (gQFinder->fSettings.fMainWindow.fMinimizeMode == kMinimizeModeTray)
		{
			NOTIFYICONDATA nid;
			nid.cbSize = sizeof (NOTIFYICONDATA);
			nid.hWnd = fWnd;
			nid.uID = IDC_TRAYICON;
			nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
			nid.uCallbackMessage = kMsgTrayIcon;
			nid.hIcon = gQFinder->fIcon;
			lstrcpyn (nid.szTip, gQFinder->fAppName.c_str(), sizeof (nid.szTip));
			Shell_NotifyIcon (NIM_ADD, &nid);
			ShowWindow (fWnd, SW_HIDE);
		}
	}
	if (wParam == SIZE_MAXIMIZED ||
		wParam == SIZE_RESTORED)
	{
		ResizeControls(kDragNone);
		GetWindowRect(fWnd, &rect);
		fOldWidth = rect.right - rect.left;
		fOldHeight = rect.bottom - rect.top;
	}
	return TRUE;
}

void MainWindow::ResizeControls(long mode, bool repaint)
{
	RECT rect;
	long top;
	long left;
	long width;
	long newWidth;
	long newHeight;

	switch (mode)
	{
	case kDragTVWidth:
		fMainToolBar.GetRect(rect);
		top = rect.bottom - rect.top + 1;
		GetClientRect (fWnd, &rect);
		switch (fLayout)
		{
		case kLayout_Standard:
			if (fLayoutParameter1 == kLayout_TreeViewShort)
			{
				fGameTreeView.Move(0, top, (long) fTVWidth, (long) fLVHeight);
				left = (long) fTVWidth + 3;
				fServerListView.Move(left, top, rect.right - left, (long) fLVHeight);
			}
			if (fLayoutParameter1 == kLayout_TreeViewLong)
			{
				RECT listRect;
				fGameTreeView.Move(0, top, (long) fTVWidth, rect.bottom - top);
				left = (long) fTVWidth + 3;
				// save old width of ServerListView
				fServerListView.GetRect(listRect);
				width = listRect.right - listRect.left + 1;
				fServerListView.Move(left, top, rect.right - left, (long) fLVHeight);
				top += (long) fLVHeight + 3;
				fLVWidth = (fLVWidth * (double) (rect.right - left + 1)) / (double) width;
				if (fLayoutParameter2 == kLayout_PlayerLeft)
				{
					fPlayerListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
					left += (long) fLVWidth + 3;
					fVariableListView.Hide();
					fVariableListView.Move(left, top, rect.right - left, rect.bottom - top);
					fVariableListView.Show();
				}
				if (fLayoutParameter2 == kLayout_PlayerRight)
				{
					fVariableListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
					left += (long) fLVWidth + 3;
					fPlayerListView.Hide();
					fPlayerListView.Move(left, top, rect.right - left, rect.bottom - top);
					fPlayerListView.Show();
				}
			}
			break;
		}
		break;
	case kDragLVHeight:
		fMainToolBar.GetRect(rect);
		top = rect.bottom - rect.top + 1;
		GetClientRect (fWnd, &rect);
		switch (fLayout)
		{
		case kLayout_Standard:
			if (fLayoutParameter1 == kLayout_TreeViewShort)
			{
				fGameTreeView.Move(0, top, (long) fTVWidth, (long) fLVHeight);
				left = (long) fTVWidth + 3;
				fServerListView.Move(left, top, rect.right - left, (long) fLVHeight);
				left = 0;
			}
			if (fLayoutParameter1 == kLayout_TreeViewLong)
			{
				left = (long) fTVWidth + 3;
				fServerListView.Move(left, top, rect.right - left, (long) fLVHeight);
			}
			top += (long) fLVHeight + 3;
			if (fLayoutParameter2 == kLayout_PlayerLeft)
			{
				fPlayerListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
				left += (long) fLVWidth + 3;
				fVariableListView.Move(left, top, rect.right - left, rect.bottom - top);
			}
			if (fLayoutParameter2 == kLayout_PlayerRight)
			{
				fVariableListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
				left += (long) fLVWidth + 3;
				fPlayerListView.Move(left, top, rect.right - left, rect.bottom - top);
			}
			break;
		}
		break;
	case kDragLVWidth:
		fMainToolBar.Move(0, 0, 0, 0);
		fMainToolBar.GetRect(rect);
		top = rect.bottom - rect.top + 1;
		GetClientRect (fWnd, &rect);
		switch (fLayout)
		{
		case kLayout_Standard:
			if (fLayoutParameter1 == kLayout_TreeViewShort)
			{
				left = 0;
			}
			if (fLayoutParameter1 == kLayout_TreeViewLong)
			{
				left = (long) fTVWidth + 3;
			}
			top += (long) fLVHeight + 3;
			if (fLayoutParameter2 == kLayout_PlayerLeft)
			{
				fPlayerListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
				left += (long) fLVWidth + 3;
				fVariableListView.Move(left, top, rect.right - left, rect.bottom - top);
			}
			if (fLayoutParameter2 == kLayout_PlayerRight)
			{
				fVariableListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
				left += (long) fLVWidth + 3;
				fPlayerListView.Move(left, top, rect.right - left, rect.bottom - top);
			}
			break;
		}
		break;
	default:
		GetWindowRect(fWnd, &rect);
		newWidth = rect.right - rect.left;
		newHeight = rect.bottom - rect.top;
		fMainToolBar.Move(0, 0, 0, 0);
		fMainToolBar.GetRect(rect);
		top = rect.bottom - rect.top + 1;
		GetClientRect (fWnd, &rect);
		switch (fLayout)
		{
		case kLayout_Standard:
			fLVHeight = (fLVHeight * (double) (newHeight - top)) / ((double) (fOldHeight - top));
			if (fLVHeight > (rect.bottom - rect.top + 1 - top - 10))
				fLVHeight = rect.bottom - rect.top + 1 - top - 10;
			if (fLayoutParameter1 == kLayout_TreeViewShort)
			{
				fGameTreeView.Move(0, top, (long) fTVWidth, (long) fLVHeight);
				left = (long) fTVWidth + 3;
				fServerListView.Move(left, top, rect.right - left, (long) fLVHeight);
				left = 0;
				top += (long) fLVHeight + 3;
				fLVWidth = (fLVWidth * (double) newWidth) / (double) fOldWidth;
				if (fLayoutParameter2 == kLayout_PlayerLeft)
				{
					fPlayerListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
					left += (long) fLVWidth + 3;
					fVariableListView.Hide();
					fVariableListView.Move(left, top, rect.right - left, rect.bottom - top);
					fVariableListView.Show();
				}
				if (fLayoutParameter2 == kLayout_PlayerRight)
				{
					fVariableListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
					left += (long) fLVWidth + 3;
					fPlayerListView.Hide();
					fPlayerListView.Move(left, top, rect.right - left, rect.bottom - top);
					fPlayerListView.Show();
				}
			}
			if (fLayoutParameter1 == kLayout_TreeViewLong)
			{
				RECT listRect;
				fGameTreeView.Move(0, top, (long) fTVWidth, rect.bottom - top);
				left = (long) fTVWidth + 3;
				// save old width of ServerListView
				fServerListView.GetRect(listRect);
				width = listRect.right - listRect.left + 1;
				fServerListView.Move(left, top, rect.right - left, (long) fLVHeight);
				top += (long) fLVHeight + 3;
				fLVWidth = (fLVWidth * (double) (rect.right - left + 1)) / (double) width;
				if (fLayoutParameter2 == kLayout_PlayerLeft)
				{
					fPlayerListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
					left += (long) fLVWidth + 3;
					fVariableListView.Hide();
					fVariableListView.Move(left, top, rect.right - left, rect.bottom - top);
					fVariableListView.Show();
				}
				if (fLayoutParameter2 == kLayout_PlayerRight)
				{
					fVariableListView.Move(left, top, (long) fLVWidth, rect.bottom - top);
					left += (long) fLVWidth + 3;
					fPlayerListView.Hide();
					fPlayerListView.Move(left, top, rect.right - left, rect.bottom - top);
					fPlayerListView.Show();
				}
			}
			break;
		}
	}
}

void MainWindow::DrawXORRect(RECT &rect)
{
	long oldMode;
	HDC hDC;

	hDC = GetDC(fWnd);
	oldMode = SetROP2(hDC, R2_XORPEN);
	Rectangle(hDC, rect.left, rect.top, rect.right, rect.bottom);
	SetROP2(hDC, oldMode);
	ReleaseDC(fWnd, hDC);
}

LRESULT MainWindow::OnNetwork(WPARAM wParam, LPARAM lParam)
{
	ServerStatus *newStatus = (ServerStatus *) wParam;
	ServerList *svrList;
	GameServer *server;
	set<ServerList>::iterator itList;
	set<GameServer>::iterator itServer;
	Parser p;
	if (newStatus->fError == 0)
	{
		long game = p.GetGameFromStatus(newStatus->fStatus);
		if (game == kGameHL && (unsigned char) newStatus->fStatus[0] == 0xFE)
		{
			// multi part packet
			newStatus->fGame = game;
			list <ServerStatus>::iterator itPacket;
			for (itPacket = fOpenPackets.begin(); itPacket != fOpenPackets.end(); itPacket++)
			{
				if (itPacket->fGame == newStatus->fGame &&
					itPacket->fStatus.find(newStatus->fStatus.substr(4,4)) == 4)
				{
					break;
				}
			}
			if (itPacket == fOpenPackets.end())
			{
				fOpenPackets.push_back(*newStatus);
				return TRUE;
			}
			newStatus->fStatus = itPacket->fStatus.substr(9) + newStatus->fStatus.substr(9);
			fOpenPackets.erase(itPacket);
		}
		if (game != -1)
		{
			svrList = gQFinder->GetServerList(gQFinder->fBroadcastID);
			if (svrList != NULL &&
				gQFinder->fBCSocket == lParam)
			{
				bool found = false;
				for (itList = gQFinder->fServerLists.begin();
					itList != gQFinder->fServerLists.end();
					itList++)
				{
					if (itList->fID != gQFinder->fBroadcastID)
					{
						if (itList->FindServerQuery(game, newStatus->fIP, newStatus->fPort))
						{
							found = true;
							break;
						}
					}
				}
				if (!found)
				{
					server = svrList->FindServerQuery(game, newStatus->fIP, newStatus->fPort);
					if (server != NULL)
					{
						p.ParseServerStatus(newStatus, server);
						if (game == kGameHL)
							gHalflifeRefresh->AddServer(server, lParam);
						AddServer(server);
					}
					else
					{
						GameServer *newServer = new GameServer(newStatus->fIP, newStatus->fPort);
						newServer->fGame = game;
						newServer->fListID = svrList->fID;
						p.ParseServerStatus(newStatus, newServer);
						svrList->fServers.insert(*newServer);
						itServer = svrList->fServers.find(*newServer);
						if (game == kGameHL)
							gHalflifeRefresh->AddServer(itServer.operator ->(), lParam);
						AddServer(itServer.operator ->());
						delete newServer;
					}
				}
			}

			for (itList = gQFinder->fServerLists.begin();
				itList != gQFinder->fServerLists.end();
				itList++)
			{
				if (itList->fSocket == lParam)
				{
					server = itList->FindServerQuery(game, newStatus->fIP, newStatus->fPort);
					if (server != NULL)
					{
						p.ParseServerStatus(newStatus, server);
						if (game == kGameHL)
							gHalflifeRefresh->AddServer(server, lParam);
						AddServer(server);
					}
					else if (itList->fID == gQFinder->fBroadcastID)
					{
						GameServer *newServer = new GameServer(newStatus->fIP, newStatus->fPort);
						newServer->fGame = game;
						newServer->fListID = itList->fID;
						p.ParseServerStatus(newStatus, newServer);
						itList->fServers.insert(*newServer);
						itServer = itList->fServers.find(*newServer);
						if (game == kGameHL)
							gHalflifeRefresh->AddServer(itServer.operator ->(), lParam);
						AddServer(itServer.operator ->());
						delete newServer;
					}
					else if (game == kGameFCRY)
					{
						svrList = gQFinder->GetServerList(gQFinder->fBroadcastID);
						if (svrList != NULL)
						{
							GameServer *newServer = new GameServer(newStatus->fIP, newStatus->fPort);
							newServer->fGame = game;
							newServer->fListID = svrList->fID;
							p.ParseServerStatus(newStatus, newServer);
							svrList->fServers.insert(*newServer);
							itServer = svrList->fServers.find(*newServer);
							AddServer(itServer.operator ->());
							delete newServer;
						}
					}
				}
			}
			MarkOldServer();
		}
		else
		{
			svrList = gQFinder->GetServerList(gQFinder->fBroadcastID);
			if (svrList != NULL &&
				gQFinder->fBCSocket == lParam)
			{
				long number = 0;
				while ((server = svrList->FindServerQuery(game, newStatus->fIP, newStatus->fPort, number)) != NULL)
				{
					if (p.CheckGameFromStatus(newStatus->fStatus, server))
					{
						p.ParseServerStatus(newStatus, server);
						AddServer(server);
						break;
					}
					number++;
				}
			}

			for (itList = gQFinder->fServerLists.begin();
				itList != gQFinder->fServerLists.end();
				itList++)
			{
				if (itList->fSocket == lParam)
				{
					long number = 0;
					while ((server = itList->FindServerQuery(game, newStatus->fIP, newStatus->fPort, number)) != NULL)
					{
						if (p.CheckGameFromStatus(newStatus->fStatus, server))
						{
							p.ParseServerStatus(newStatus, server);
							AddServer(server);
							break;
						}
						number++;
					}
				}
			}
			MarkOldServer();
		}
	}
	else
	{
		char message[100];
		sprintf(message, "Net %d %d %s:%s",
			newStatus->fError, lParam,
			newStatus->fIPStr.c_str(), newStatus->fPortStr.c_str());
		gLogFile->DebugMessage(message);
		if (newStatus->fError == WSAECONNRESET)
		{
			server = NULL;
			for (itList = gQFinder->fServerLists.begin();
				itList != gQFinder->fServerLists.end();
				itList++)
			{
				if (itList->fSocket == lParam)
				{
					server = itList->FindServerQuery(-1, newStatus->fIP, newStatus->fPort);
					if (server != NULL)
					{
						if (fCurrentServer &&
							fCurrentServer->fGame == server->fGame &&
							fCurrentServer->fIP == server->fIP &&
							fCurrentServer->fQueryPort == server->fQueryPort)
						{
							SetCurrentServer(NULL);
						}
						RemoveServer(server);
						if (itList->fID == gQFinder->fBroadcastID)
							itList->fServers.erase(*server);
						else
							server->fFound = false;
						break;
					}
				}
			}
		}
	}
	delete newStatus;

	return TRUE;
}

void MainWindow::JoinGame()
{
	string path;
	string cmdline;

	if (!fCurrentServer)
		return;
	cmdline = gQFinder->fSettings.fGame.fInfos[fCurrentServer->fGame].fPath;
	cmdline += "\\";
	cmdline += gQFinder->fSettings.fGame.fInfos[fCurrentServer->fGame].fExe;
	path = cmdline;
	cmdline += " ";
	if (fCurrentServer->fGame == kGameFCRY)
		cmdline += "\"";
	cmdline += gGames[fCurrentServer->fGame].fConnect;
	cmdline += " ";
	cmdline += fCurrentServer->fIPStr + ":" + fCurrentServer->fPortStr;
	if (fCurrentServer->fGame == kGameFCRY)
		cmdline += "\"";
	cmdline += " ";
	if (fCurrentServer->fGameDir.length() > 0 &&
		gGames[fCurrentServer->fGame].fGameDir.length() > 0)
	{
		cmdline += gGames[fCurrentServer->fGame].fGameDir;
		cmdline += " ";
		cmdline += fCurrentServer->fGameDir;
		cmdline += " ";
	}
	cmdline += gQFinder->fSettings.fGame.fInfos[fCurrentServer->fGame].fParameters;

	if (fCurrentServer->fCntPlayers == fCurrentServer->fMaxPlayers)
		if (MessageBox (fWnd, "Server is full, join nevertheless?", "Join game", MB_YESNO) == IDNO)
			return;
	if (fCurrentServer->fNeedPass)
	{
		string pass;
		INT_PTR result;
		DlgPassword *dialog = new DlgPassword(fWnd);
		result = dialog->Create(&pass);
		delete dialog;
		if (result == IDCANCEL)
			return;
		cmdline += " ";
		if (fCurrentServer->fGame == kGameFCRY)
			cmdline += "\"";
		cmdline += gGames[fCurrentServer->fGame].fPassword;
		cmdline += " ";
		cmdline += pass;
		if (fCurrentServer->fGame == kGameFCRY)
			cmdline += "\"";
	}
	chdir(gQFinder->fSettings.fGame.fInfos[fCurrentServer->fGame].fPath.c_str());

	STARTUPINFO SUInfo;
	PROCESS_INFORMATION PInfo;

	SUInfo.cb = sizeof(CREATESTRUCT);
	SUInfo.lpReserved = NULL;
	SUInfo.lpDesktop = NULL;
	SUInfo.lpTitle = NULL;
	SUInfo.dwFlags = 0;
	SUInfo.cbReserved2 = 0;
	SUInfo.lpReserved2 = NULL;

	if (CreateProcess((LPSTR) path.c_str(), (LPSTR) cmdline.c_str(), NULL, NULL, TRUE, 0, NULL, NULL, &SUInfo, &PInfo) == 0)
	{
		MessageBox (fWnd, "Game could not be started!\n\nCheck the game settings in Preferences!", gQFinder->fAppName.c_str(), MB_ICONERROR);
	}
	
	chdir(gQFinder->fWorkingDir.c_str());
}

void MainWindow::UpdateAllGames()
{
	SetOldServer(fCurrentServer);
	SetCurrentServer(NULL);
	fOpenPackets.clear();
	ServerList *svrList = gQFinder->GetServerList(gQFinder->fBroadcastID);
	fServerListView.DeleteAllItems();
	fPlayerListView.DeleteAllItems();
	fVariableListView.DeleteAllItems();
	svrList->fServers.clear();
	gSocketWindow->AddSocketEvent(gQFinder->fBCSocket, FD_READ, kMsgNetwork);
	gSocketWindow->AddSocketEvent(svrList->fSocket, FD_READ, kMsgNetwork);

	if (gQFinder->fBCPorts.size() > 0)
	{
		int err;
		struct sockaddr_in toAddr;
		set<BCPort>::iterator port;
		long index;
		string str;

		toAddr.sin_family = AF_INET;
		toAddr.sin_addr.S_un.S_addr = INADDR_BROADCAST;

		for (port = gQFinder->fBCPorts.begin(); port != gQFinder->fBCPorts.end(); port++)
		{
			toAddr.sin_port = htons(port->fPort);
			for (index = 0; index < (long) gGames[port->fGame].fQueryStrings.size(); index++)
			{
				str = gGames[port->fGame].fQueryStrings[index];
				if (sendto(gQFinder->fBCSocket, str.c_str(), str.length(), 0, (sockaddr *) &toAddr, sizeof(sockaddr)) == SOCKET_ERROR)
				{
					err = WSAGetLastError();
				}
			}
		}
	}
	svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
	gSocketWindow->AddSocketEvent(svrList->fSocket, FD_READ, kMsgNetwork);
	set<GameServer>::iterator itServer;
	for (itServer = svrList->fServers.begin();
		 itServer != svrList->fServers.end();
		 itServer++)
	{
		int err;
		struct sockaddr_in toAddr;
		long index;
		string str;

		toAddr.sin_family = AF_INET;
		toAddr.sin_addr.S_un.S_addr = htonl(itServer->fIP);
		toAddr.sin_port = htons(itServer->fQueryPort);
		for (index = 0; index < (long) gGames[itServer->fGame].fQueryStrings.size(); index++)
		{
			str = gGames[itServer->fGame].fQueryStrings[index];
			if (sendto(svrList->fSocket, str.c_str(), str.length(), 0, (sockaddr *) &toAddr, sizeof(sockaddr)) == SOCKET_ERROR)
			{
				err = WSAGetLastError();
			}
		}
	}
}

void MainWindow::UpdateSelectedGame()
{
	set<GameServer>::iterator itServer;
	set<GameServer>::iterator itServerDel;

	SetOldServer(fCurrentServer);
	SetCurrentServer(NULL);
	fOpenPackets.clear();
	ServerList *svrList = gQFinder->GetServerList(gQFinder->fBroadcastID);
	fServerListView.DeleteAllItems();
	fPlayerListView.DeleteAllItems();
	fVariableListView.DeleteAllItems();

	for (itServer = svrList->fServers.begin();
		 itServer != svrList->fServers.end();)
	{
		if (itServer->fGame == fCurrentGame)
		{
			itServerDel = itServer;
			itServer++;
			svrList->fServers.erase(itServerDel);
		}
		else
			itServer++;
	}

	gSocketWindow->AddSocketEvent(gQFinder->fBCSocket, FD_READ, kMsgNetwork);
	gSocketWindow->AddSocketEvent(svrList->fSocket, FD_READ, kMsgNetwork);

	if (gQFinder->fBCPorts.size() > 0)
	{
		int err;
		struct sockaddr_in toAddr;
		set<BCPort>::iterator port;
		long index;
		string str;

		toAddr.sin_family = AF_INET;
		toAddr.sin_addr.S_un.S_addr = INADDR_BROADCAST;

		for (port = gQFinder->fBCPorts.begin(); port != gQFinder->fBCPorts.end(); port++)
		{
			if (port->fGame == fCurrentGame)
			{
				toAddr.sin_port = htons(port->fPort);
				for (index = 0; index < (long) gGames[port->fGame].fQueryCnt; index++)
				{
					str = gGames[port->fGame].fQueryStrings[index];
					if (sendto(gQFinder->fBCSocket, str.c_str(), str.length(), 0, (sockaddr *) &toAddr, sizeof(sockaddr)) == SOCKET_ERROR)
					{
						err = WSAGetLastError();
					}
				}
			}
		}
	}
	svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
	gSocketWindow->AddSocketEvent(svrList->fSocket, FD_READ, kMsgNetwork);
	for (itServer = svrList->fServers.begin();
		 itServer != svrList->fServers.end();
		 itServer++)
	{
		int err;
		struct sockaddr_in toAddr;
		long index;
		string str;

		if (itServer->fGame == fCurrentGame)
		{
			itServer->fFound = false;
			toAddr.sin_family = AF_INET;
			toAddr.sin_addr.S_un.S_addr = htonl(itServer->fIP);
			toAddr.sin_port = htons(itServer->fQueryPort);
			for (index = 0; index < (long) gGames[itServer->fGame].fQueryCnt; index++)
			{
				str = gGames[itServer->fGame].fQueryStrings[index];
				if (sendto(svrList->fSocket, str.c_str(), str.length(), 0, (sockaddr *) &toAddr, sizeof(sockaddr)) == SOCKET_ERROR)
				{
					err = WSAGetLastError();
				}
			}
		}
	}
}

void MainWindow::UpdateSingleServer()
{
	int err;
	struct sockaddr_in toAddr;
	long index;
	string str;
	ServerList *svrList;

	SetOldServer(fCurrentServer);
	fOpenPackets.clear();
	svrList = gQFinder->GetServerList(fCurrentServer->fListID);
	gSocketWindow->AddSocketEvent(svrList->fSocket, FD_READ, kMsgNetwork);

	toAddr.sin_family = AF_INET;
	toAddr.sin_addr.S_un.S_addr = htonl(fCurrentServer->fIP);
	toAddr.sin_port = htons(fCurrentServer->fQueryPort);
	for (index = 0; index < (long) gGames[fCurrentServer->fGame].fQueryCnt; index++)
	{
		str = gGames[fCurrentServer->fGame].fQueryStrings[index];
		if (sendto(svrList->fSocket, str.c_str(), str.length(), 0, (sockaddr *) &toAddr, sizeof(sockaddr)) == SOCKET_ERROR)
		{
			err = WSAGetLastError();
		}
	}
}

void MainWindow::Preferences()
{
	if (gQFinder->fSettings.fHotkeys.minimizeActive)
		UnregisterHotKey(fWnd, gQFinder->fSettings.fHotkeys.minimizeId);
	// Spaltenbreiten sichern
	for (int i = 0; i < (int) gQFinder->fSettings.fMainWindow.fServerColumns.size(); i++)
	{
		gQFinder->fSettings.fMainWindow.fServerColumns[i].fWidth = fServerListView.GetColumnWidth(gQFinder->fSettings.fMainWindow.fServerColumns[i].fPosition);
	}
	for (int i = 0; i < (int) gQFinder->fSettings.fMainWindow.fPlayerColumns.size(); i++)
	{
		gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fWidth = fPlayerListView.GetColumnWidth(gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fPosition);
	}
	for (int i = 0; i < (int) gQFinder->fSettings.fMainWindow.fVariableColumns.size(); i++)
	{
		gQFinder->fSettings.fMainWindow.fVariableColumns[i].fWidth = fVariableListView.GetColumnWidth(gQFinder->fSettings.fMainWindow.fVariableColumns[i].fPosition);
	}

	DlgPreferences *dialog = new DlgPreferences(fWnd);
	dialog->Create();
	delete dialog;
	chdir(gQFinder->fWorkingDir.c_str());

	if (gQFinder->fSettings.fChanged & kChangedLayout)
	{
		fLayout = gQFinder->fSettings.fMainWindow.fLayout;
		fLayoutParameter1 = gQFinder->fSettings.fMainWindow.fLayoutParameter1;
		fLayoutParameter2 = gQFinder->fSettings.fMainWindow.fLayoutParameter2;
	}
	if (gQFinder->fSettings.fChanged & kChangedToolBar)
	{
		fMainToolBar.Rebuild();
		fMainToolBar.ChangeBitmap(ID_ACTIONS_UPDATESELECTEDGAME, kToolBarIndexRefreshGame + fCurrentGame);
		gQFinder->fSettings.fChanged |= kChangedLayout;
	}
	if (gQFinder->fSettings.fChanged & kChangedServerList)
	{
		LVITEM item;
		GameServer *server;
		set<GameServer>::iterator itServer;
		int index;

		if (fServerListView.GetItemCount() > 0)
		{
			item.mask = LVIF_PARAM;
			if (fServerListView.GetSelectedItem(&item))
			{
				server = (GameServer *) item.lParam;
			}
			else
			{
				item.iItem = 0;
				item.iSubItem = 0;
				fServerListView.GetItem(&item);
				server = (GameServer *) item.lParam;
			}
			fServerListView.DeleteAllItems();
			fServerListView.Rebuild();
			if (server->fListID == gQFinder->fBroadcastID ||
				server->fListID == gQFinder->fFavoritesID)
			{
				ServerList *svrList = gQFinder->GetServerList(gQFinder->fBroadcastID);
				if (svrList != NULL)
				{
					for (itServer = svrList->fServers.begin();
						itServer != svrList->fServers.end();
						itServer++)
					{
						if (itServer->fFound)
							fServerListView.AddServer(itServer.operator ->());
					}
				}
				svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
				if (svrList != NULL)
				{
					for (itServer = svrList->fServers.begin();
						itServer != svrList->fServers.end();
						itServer++)
					{
						if (itServer->fFound)
							fServerListView.AddServer(itServer.operator ->());
					}
				}
			}
			else
			{
				ServerList *svrList = gQFinder->GetServerList(server->fListID);
				if (svrList != NULL)
				{
					for (itServer = svrList->fServers.begin();
						itServer != svrList->fServers.end();
						itServer++)
					{
						if (itServer->fFound)
							fServerListView.AddServer(itServer.operator ->());
					}
				}
			}
			index = fServerListView.FindServer(server);
			fServerListView.SetSelectedItem(index);
		}
		else
		{
			fServerListView.Rebuild();
		}
	}
	if (gQFinder->fSettings.fChanged & kChangedPlayerList)
	{
		LVITEM item;
		GameServer *server;

		if (fServerListView.GetItemCount() > 0)
		{
			item.mask = LVIF_PARAM;
			if (fServerListView.GetSelectedItem(&item))
			{
				server = (GameServer *) item.lParam;
			}
			else
			{
				item.iItem = 0;
				item.iSubItem = 0;
				fServerListView.GetItem(&item);
				server = (GameServer *) item.lParam;
			}
			fPlayerListView.DeleteAllItems();
			fPlayerListView.Rebuild();
			vector<Player>::iterator itPlayer;
			for (itPlayer = server->fPlayers.begin();
				 itPlayer != server->fPlayers.end();
				 itPlayer++)
			{
				fPlayerListView.AddPlayer(itPlayer.operator ->());
			}
		}
		else
		{
			fPlayerListView.Rebuild();
		}
	}
	if (gQFinder->fSettings.fChanged & kChangedVariableList)
	{
		LVITEM item;
		GameServer *server;

		if (fServerListView.GetItemCount() > 0)
		{
			item.mask = LVIF_PARAM;
			if (fServerListView.GetSelectedItem(&item))
			{
				server = (GameServer *) item.lParam;
			}
			else
			{
				item.iItem = 0;
				item.iSubItem = 0;
				fServerListView.GetItem(&item);
				server = (GameServer *) item.lParam;
			}
			fVariableListView.DeleteAllItems();
			fVariableListView.Rebuild();
			list<StringVariable>::iterator itVariable;
			for (itVariable = server->fVariables.begin();
				 itVariable != server->fVariables.end();
				 itVariable++)
			{
				fVariableListView.AddVariable(itVariable.operator ->());
			}
		}
		else
		{
			fVariableListView.Rebuild();
		}
	}
	if (gQFinder->fSettings.fChanged & kChangedGameList)
	{
		fGameTreeView.Rebuild();
		gQFinder->fSettings.fChanged |= kChangedLayout;
	}
	if (gQFinder->fSettings.fChanged & kChangedLayout &&
		gQFinder->fSettings.fMainWindow.fMode != kWindowModeMinimized)
	{
		ResizeControls(kDragNone);
	}
	gQFinder->fSettings.fChanged = 0;

	if (gQFinder->fSettings.fHotkeys.minimizeActive)
		RegisterHotKey(fWnd,
			gQFinder->fSettings.fHotkeys.minimizeId,
			gQFinder->fSettings.fHotkeys.minimizeModifier,
			gQFinder->fSettings.fHotkeys.minimizeVirtualKey);
}

void MainWindow::AddServer(GameServer *server)
{
	if ((fCurrentGame == kGameAll || fCurrentGame == server->fGame) &&
		server->fFound)
	{
/*		bool complete = true;
		if (server->fGame == kGameHL)
		{
			for (int packet = 0; packet < gGames[kGameHL].fQueryCnt; packet++)
				if (server->fGotPacket[packet] == false)
					complete = false;
		}
		if (complete)
*/			fServerListView.AddServer(server);
	}
}

void MainWindow::RemoveServer(GameServer *server)
{
	if ((fCurrentGame == kGameAll || fCurrentGame == server->fGame))
	{
		int item = fServerListView.FindServer(server);
		if (item != -1)
		{
			fServerListView.DeleteItem(item);
			fPlayerListView.DeleteAllItems();
			fVariableListView.DeleteAllItems();
		}
	}
}

void MainWindow::MarkOldServer()
{
	long item;

	item = fServerListView.FindServer(&fOldServer);
	if (item != -1)
	{
		fServerListView.SetSelectedItem(item);
	}
}

LRESULT APIENTRY MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gMainWindow->WindowProc(hWnd, message, wParam, lParam);
}

MainWindow *gMainWindow;