/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#include "Settings.h"
#include "MainToolBar.h"
#include "GameTreeView.h"
#include "ServerListView.h"
#include "PlayerListView.h"
#include "VariableListView.h"
#include "SocketWindow.h"
#include "ServerStatus.h"

const long kDragNone = 0;
const long kDragTVWidth = 1;
const long kDragLVHeight = 2;
const long kDragLVWidth = 3;

const long kMinWidth = 500;
const long kMinHeight = 400;

static const int kMsgTrayIcon = WM_USER + 0;

class MainWindow
{
public:
	MainWindow();
	~MainWindow();
	void Create();
	void Show();
	int SendMsg (UINT message, WPARAM wParam, LPARAM lParam);
	LRESULT WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	MainToolBar *GetMainToolBar();
	ServerListView *GetServerListView();
	PlayerListView *GetPlayerListView();
	VariableListView *GetVariableListView();
	void SetCurrentGame(long game);
	long GetCurrentGame();
	void SetCurrentServer(GameServer *server);
	GameServer *GetCurrentServer();
	void SetOldServer(GameServer *server);
	void MarkOldServer();
private:
	LRESULT OnCommand(WPARAM wParam, LPARAM lParam);
	LRESULT OnCreate(HWND hWnd);
	LRESULT OnLeftButtonDown(WPARAM wParam, LPARAM lParam);
	LRESULT OnLeftButtonUp(WPARAM wParam, LPARAM lParam);
	LRESULT OnMouseMove(WPARAM wParam, LPARAM lParam);
	LRESULT OnSizing(WPARAM wParam, LPARAM lParam);
	LRESULT OnSize(WPARAM wParam, LPARAM lParam);
	LRESULT OnNetwork(WPARAM wParam, LPARAM lParam);
	void ResizeControls(long mode, bool repaint = false);
	void DrawXORRect(RECT &Rect);

	void JoinGame();
	void UpdateAllGames();
	void UpdateSelectedGame();
	void UpdateSingleServer();
	void Preferences();
	void AddServer(GameServer *server);
	void RemoveServer(GameServer *server);

	HWND fWnd;

	MainToolBar fMainToolBar;
	GameTreeView fGameTreeView;
	ServerListView fServerListView;
	PlayerListView fPlayerListView;
	VariableListView fVariableListView;

	long fCurrentGame;
	GameServer *fCurrentServer;
	GameServer fOldServer;
	list <ServerStatus> fOpenPackets;

    long fLayout;
	long fLayoutParameter1;
	long fLayoutParameter2;

	double fTVWidth;
	double fLVHeight;
	double fLVWidth;

	long fDragMode;
	long fDragStart;
	long fDragLast;
	long fDragDelta;

	long fOldWidth;
	long fOldHeight;
};

LRESULT APIENTRY MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

extern MainWindow *gMainWindow;

#endif //_MAINWINDOW_H_
