/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#ifndef _PARSER_H_
#define _PARSER_H_

#include "GameServer.h"
#include "ServerStatus.h"
#include "BitStream.h"

static const long kGameTypeFFA = 0;
static const long kGameType1V1 = 1;
static const long kGameTypeSP = 2;
static const long kGameTypeTDM = 3;
static const long kGameTypeCTF = 4;

class Parser
{
public:
	void ParseServerStatus(ServerStatus *state, GameServer *server);
	long GetGameFromStatus(string &status);
	bool CheckGameFromStatus(string &status, GameServer *server);
private:
	long ParseVariables(long posBuf, string &status, GameServer *server);
	long ParsePlayers(long posBuf, string &status, GameServer *server, bool containsFrags);
	void ParseQuakeworldStatus(string &status, GameServer *server);
	void ParseQuake2Status(string &status, GameServer *server);
	void ParseQuake3Status(string &status, GameServer *server);
	void ParseHalfLifeStatus(string &status, GameServer *server);
	void ParseUnrealTournamentStatus(string &status, GameServer *server);
	void ParseUnrealTournament2003Status(string &status, GameServer *server);
	void ParseSoldierOfFortuneStatus(string &status, GameServer *server);
	void ParseTribesStatus(string &status, GameServer *server);
	void ParseTribes2Status(string &status, GameServer *server);
	void ParseFarCryStatus(string &status, GameServer *server);
	string DecodeFarCryPacket(string & in);
	string ReadFarCryString(cBitStream & s);
};

static const char LetterCodeQuakeworld[256] = {
	'',' ',' ',' ',' ','',' ',' ',' ',' ',' ',' ',' ',' ','','',
	'[',']','0','1','2','3','4','5','6','7','8','9','',' ',' ',' ',
	' ','!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/',
	'0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','\?',
	'@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
	'P','Q','R','S','T','U','V','W','X','Y','Z','[','\\',']','^','_',
	'`','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
	'p','q','r','s','t','u','v','w','x','y','z','{','','}','~','<',
	' ',' ',' ',' ',' ','',' ',' ',' ',' ',' ',' ',' ',' ','','',
	'[',']','0','1','2','3','4','5','6','7','8','9','',' ',' ',' ',
	' ','!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/',
	'0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','\?',
	'@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
	'P','Q','R','S','T','U','V','W','X','Y','Z','[','\\',']','^','_',
	'`','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
	'p','q','r','s','t','u','v','w','x','y','z','{','','}','~','<'
};

static const char LetterCodeQuake2[256] = {
	'',' ',' ',' ',' ','',' ',' ',' ',' ',' ',' ',' ',' ','','',
	'[',']','0','1','2','3','4','5','6','7','8','9','',' ',' ',' ',
	' ','!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/',
	'0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','\?',
	'@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
	'P','Q','R','S','T','U','V','W','X','Y','Z','[','\\',']','^','_',
	'`','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
	'p','q','r','s','t','u','v','w','x','y','z','{','','}','~','<',
	' ',' ',' ',' ',' ','',' ',' ',' ',' ',' ',' ',' ',' ','','',
	'[',']','0','1','2','3','4','5','6','7','8','9','',' ',' ',' ',
	' ','!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/',
	'0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','\?',
	'@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
	'P','Q','R','S','T','U','V','W','X','Y','Z','[','\\',']','^','_',
	'`','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
	'p','q','r','s','t','u','v','w','x','y','z','{','','}','~','<'
};

static const char LetterCodeQuake3[256] = {
	' ',' ',' ',' ',' ','',' ',' ',' ',' ',' ',' ',' ',' ','','',
	'[',']',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ','',' ',' ',' ',
	' ','!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/',
	'0','1','2','3','4','5','6','7','8','9',' ',';','<','=','>','\?',
	'@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
	'P','Q','R','S','T','U','V','W','X','Y','Z','[','\\',']','^',' ',
	'`','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o',
	'p','q','r','s','t','u','v','w','x','y','z','{','','}','\"','<',
	' ',' ',' ',' ',' ','',' ',' ',' ',' ',' ',' ',' ',' ','','',
	'[',']','0','1','2','3','4','5','6','7','8','','',' ',' ',' ',
	' ','!','\"','#','$','%','&','\'','(',')','*','+',',','-','.','/',
	'0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','\?',
	'@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
	'P','Q','R','S','T','U','V','W','X','Y','Z','[','\\',']','^','_',
	'`','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
	'P','Q','R','S','T','U','V','W','X','Y','Z','{','','}','\"',' '
};

#endif //_PARSER_H_