/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "PlayerListView.h"
#include "QFinder.h"

int CALLBACK PlayerListView_CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);

PlayerListView::PlayerListView()
: ListView()
{
}

PlayerListView::~PlayerListView()
{
}

void PlayerListView::Create(HWND hParent)
{
	ListView::Create(hParent, IDC_PLAYER_LISTVIEW);
	SetStyleFullRowSelect();
	COLUMNINFO newCol;
	int i;
	int j;

	for (i = 0; i < (int) gQFinder->fSettings.fMainWindow.fPlayerColumns.size(); i++)
	{
		newCol.fVisible = gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fVisible;
		newCol.fPosition = gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fPosition;
		newCol.fWidth = gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fWidth;
		newCol.fSort = gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fSort;
		newCol.fFormat = gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fFormat;
		newCol.fTitle = gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fTitle;
		fColumns.push_back(newCol);
	}

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		for (j = 0; j < (int) fColumns.size(); j++)
		{
			if (fColumns[j].fPosition == i)
				break;
		}
		AddColumn(fColumns[j].fPosition, fColumns[j].fTitle, fColumns[j].fFormat, fColumns[j].fWidth, fColumns[j].fPosition);
	}
}

void PlayerListView::Rebuild()
{
	int i;
	int j;

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		if (fColumns[i].fVisible)
			DeleteColumn(0);
	}
	fColumns.clear();

	for (i = 0; i < (int) gQFinder->fSettings.fMainWindow.fPlayerColumns.size(); i++)
	{
		if (gQFinder->fSettings.fMainWindow.fPlayerColumns[i].fVisible)
		{
			fColumns.push_back(gQFinder->fSettings.fMainWindow.fPlayerColumns[i]);
		}
	}

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		for (j = 0; j < (int) fColumns.size(); j++)
		{
			if (fColumns[j].fPosition == i)
				break;
		}
		AddColumn(fColumns[j].fPosition, fColumns[j].fTitle, fColumns[j].fFormat, fColumns[j].fWidth, fColumns[j].fPosition);
	}
}

void PlayerListView::AddPlayer (Player *player)
{
	vector<string> itemStrings;
	string strItem;
	int item;
	long cnt;
	long colItem;
	char szHlp[10];

	strItem = player->fName;
	itemStrings.push_back(strItem);
	strItem = itoa(player->fFrags, szHlp, 10);
	itemStrings.push_back(strItem);
	strItem = itoa(player->fPing, szHlp, 10);
	itemStrings.push_back(strItem);


	colItem = 0;
	for (cnt = 0; cnt < (int) fColumns.size(); cnt++)
	{
		if (fColumns[cnt].fVisible && fColumns[cnt].fPosition == 0)
		{
			colItem = cnt;
			break;
		}
	}
	item = AddItem(itemStrings[colItem], 0, (LPARAM) player);
	for (cnt = 0; cnt < (int) itemStrings.size(); cnt++)
	{
		if (cnt == colItem)
			continue;
		AddSubItem(itemStrings[cnt], item, fColumns[cnt].fPosition);
	}
}

void PlayerListView::Sort()
{
	vector<COLUMNINFO>::iterator itColumn;
	bool sorted = false;
	long subItem;

	for (itColumn = gQFinder->fSettings.fMainWindow.fPlayerColumns.begin();
		 itColumn != gQFinder->fSettings.fMainWindow.fPlayerColumns.end();
		 itColumn++)
	{
		if (itColumn->fSort != kColumnSortNone)
		{
			subItem = itColumn->fPosition;
			sorted = true;
			break;
		}
	}
	if (!sorted)
	{
		subItem = 0;
		for (itColumn = gQFinder->fSettings.fMainWindow.fPlayerColumns.begin();
			itColumn != gQFinder->fSettings.fMainWindow.fPlayerColumns.end();
			itColumn++)
		{
			if (itColumn->fPosition == 0)
			{
				itColumn->fSort = kColumnSortAscending;
				break;
			}
		}
	}
	SortItems(PlayerListView_CompareFunc, subItem);
}

LRESULT PlayerListView::OnNotify (LPARAM lParam)
{
	NMLISTVIEW *nmListView = (NMLISTVIEW *) lParam;

	switch (nmListView->hdr.code)
	{
	case LVN_COLUMNCLICK:
		{
			vector<COLUMNINFO>::iterator itColumn;
			bool newCol = false;
			for (itColumn = gQFinder->fSettings.fMainWindow.fPlayerColumns.begin();
				 itColumn != gQFinder->fSettings.fMainWindow.fPlayerColumns.end();
				 itColumn++)
			{
				if (itColumn->fSort != kColumnSortNone)
				{
					if (itColumn->fPosition != nmListView->iSubItem)
					{
						itColumn->fSort = kColumnSortNone;
						newCol = true;
						break;
					}
					else
					{
						if (itColumn->fSort == kColumnSortAscending)
							itColumn->fSort = kColumnSortDescending;
						else
							itColumn->fSort = kColumnSortAscending;
					}
				}
			}
			if (newCol)
			{
				for (itColumn = gQFinder->fSettings.fMainWindow.fPlayerColumns.begin();
					 itColumn != gQFinder->fSettings.fMainWindow.fPlayerColumns.end();
					 itColumn++)
				{
					if (itColumn->fPosition == nmListView->iSubItem)
					{
						itColumn->fSort = kColumnSortAscending;
						break;
					}
				}
			}
			SortItems(PlayerListView_CompareFunc, nmListView->iSubItem);
		}
		break;
	case LVN_ITEMCHANGED:
		if (nmListView->uNewState == 3)
		{
		}
		break;
	}
	return 0;
}

int CALLBACK PlayerListView_CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	Player *paramPlayer1 = (Player *) lParam1;
	Player *paramPlayer2 = (Player *) lParam2;
	long subItem = lParamSort;
	int retVal = 0;
	vector<COLUMNINFO>::iterator itColumn;
	long ixCol = 0;

	for (itColumn = gQFinder->fSettings.fMainWindow.fPlayerColumns.begin();
		 itColumn != gQFinder->fSettings.fMainWindow.fPlayerColumns.end();
		 itColumn++)
	{
		if (itColumn->fPosition == subItem)
		{
			break;
		}
		ixCol++;
	}
	switch (ixCol)
	{
	case kColumnPlayerName:
		retVal = paramPlayer1->fName.compare(paramPlayer2->fName);
		break;
	case kColumnPlayerFrags:
		if (paramPlayer1->fFrags < paramPlayer2->fFrags)
			retVal = -1;
		else
			if (paramPlayer1->fFrags > paramPlayer2->fFrags)
				retVal = 1;
			else
				retVal = 0;
		break;
	case kColumnPlayerPing:
		if (paramPlayer1->fPing < paramPlayer2->fPing)
			retVal = -1;
		else
			if (paramPlayer1->fPing > paramPlayer2->fPing)
				retVal = 1;
			else
				retVal = 0;
		break;
	}
	if (itColumn->fSort == kColumnSortAscending)
		return retVal;
	else
		return -retVal;
}
