/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

// QFinder.cpp : Definiert den Einsprungpunkt fr die Anwendung.
//

#include "StdAfx.h"
#include "Qfinder.h"
#include "MainWindow.h"
#include "Game.h"
#include "HalflifeRefresh.h"
#include "LogFile.h"

const int kVersionMajor = 2;
const int kVersionMinor = 1;
const int kVersionMicro = 8;
const char kVersionSuffix[] = "";

QFinder::QFinder (HINSTANCE hInstance)
{
	char str[256];

	fInstance = hInstance;
	fIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_QFINDER));
	fAppName = "QFinder";
	GetCurrentDirectory(255, str);
	fWorkingDir = str;
	if (kVersionMinor != -1 &&
		kVersionMicro != -1)
	{
		sprintf(str, "%d.%d.%d%s", kVersionMajor, kVersionMinor, kVersionMicro, kVersionSuffix);
	}
	else
	{
		sprintf(str, "%d%s", kVersionMajor, kVersionSuffix);
	}
	fVersion = str;
	gLogFile = new LogFile(fWorkingDir, 0);
}

QFinder::~QFinder()
{
	delete gLogFile;
}

void QFinder::Init()
{
	WSADATA wsadata;
	WSAStartup(0x0202, &wsadata);
	Game *game;

	for (int i = 0; i <= kGameCount; i++)
	{
		game = new Game(i);
		gGames.push_back(*game);
		delete game;
	}

	fServerLists.insert(ServerList(kServerListTypeBroadcast, "Broadcast"));
	fServerLists.insert(ServerList(kServerListTypeFavorites, "Favorities"));

	fSettings.Load();

	INITCOMMONCONTROLSEX icex;
	// Ensure that the common control DLL is loaded.
	icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
	icex.dwICC  = ICC_BAR_CLASSES | ICC_LISTVIEW_CLASSES | ICC_TREEVIEW_CLASSES | ICC_UPDOWN_CLASS | ICC_INTERNET_CLASSES;
	InitCommonControlsEx(&icex);

	int err;
	int broad = 1;
	fBCSocket = socket(AF_INET, SOCK_DGRAM, 0);
	if (fBCSocket == SOCKET_ERROR)
	{
		err = WSAGetLastError();
	}
	if (setsockopt(fBCSocket, SOL_SOCKET, SO_BROADCAST, (char *) &broad, sizeof(int)) == SOCKET_ERROR)
	{
		err = WSAGetLastError();
	}

	set<ServerList>::iterator itList;
	for (itList = fServerLists.begin(); itList != fServerLists.end(); itList++)
	{
		itList->LoadFromFile();
	}

	gHalflifeRefresh = new HalflifeRefresh();

	gMainWindow = new MainWindow();
	gMainWindow->Create();
	gMainWindow->Show();
}

int QFinder::Run()
{
	MSG msg;
	int ret;

	while ((ret = GetMessage (&msg, NULL, 0, 0)) != 0)
	{
		if (ret == -1)
			break;
		TranslateMessage (&msg);
		DispatchMessage (&msg);
	}
	gQFinder->fSettings.Save();
	ServerList * svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
	svrList->SaveToFile();
	return msg.wParam;
}

ServerList *QFinder::GetServerList(long listID)
{
	set<ServerList>::iterator itList;
	for (itList = fServerLists.begin(); itList != fServerLists.end(); itList++)
	{
		if (itList->fID == listID)
		{
			return itList.operator ->();
		}
	}
	return NULL;
}


QFinder *gQFinder;

int APIENTRY WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	gQFinder = new QFinder(hInstance);

	gQFinder->Init();
	return gQFinder->Run();
}
