/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "ServerList.h"
#include "QFinder.h"
#include "Game.h"

#include <fstream>

long ServerList::fCount = 0;

ServerList::ServerList(long type, string name)
{
	int err;

	fID = fCount;
	fCount++;
	fType = type;
	fName = name;
	fHasChanged = false;
	fSocket = socket(AF_INET, SOCK_DGRAM, 0);
	if (fSocket == SOCKET_ERROR)
	{
		err = WSAGetLastError();
	}
	if (fType == kServerListTypeBroadcast)
		gQFinder->fBroadcastID = fID;
	if (fType == kServerListTypeFavorites)
		gQFinder->fFavoritesID = fID;
}

ServerList::~ServerList()
{
	set<GameServer>::iterator server;
	for (server = fServers.begin(); server != fServers.end(); server++)
	{
	}
	fServers.clear();
}

GameServer *ServerList::FindServer(long game, unsigned long ip, unsigned short port, long number)
{
	set<GameServer>::iterator server;
	if (game != -1)
		for (server = fServers.begin(); server != fServers.end(); server++)
		{
			if (server->fGame == game &&
				server->fIP == ip &&
				server->fPort == port)
			{
				return server.operator ->();
			}
		}
	else
		for (server = fServers.begin(); server != fServers.end(); server++)
		{
			if (server->fIP == ip &&
				server->fPort == port)
			{
				if (number == 0)
					return server.operator ->();
				number--;
			}
		}
	return NULL;
}

GameServer *ServerList::FindServerQuery(long game, unsigned long ip, unsigned short queryPort, long number)
{
	set<GameServer>::iterator server;
	if (game != -1)
		for (server = fServers.begin(); server != fServers.end(); server++)
		{
			if (server->fGame == game &&
				server->fIP == ip &&
				server->fQueryPort == queryPort)
			{
				return server.operator ->();
			}
		}
	else
		for (server = fServers.begin(); server != fServers.end(); server++)
		{
			if (server->fIP == ip &&
				server->fQueryPort == queryPort)
			{
				if (number == 0)
					return server.operator ->();
				number--;
			}
		}
	return NULL;
}

void ServerList::LoadFromFile()
{
	switch (fType)
	{
	case kServerListTypeBroadcast:
		break;
	case kServerListTypeFavorites:
		LoadFavorites();
		break;
	case kServerListTypeSimple:
		break;
	case kServerListTypeMaster:
		break;
	}
}

void ServerList::SaveToFile()
{
	switch (fType)
	{
	case kServerListTypeBroadcast:
		break;
	case kServerListTypeFavorites:
		SaveFavorites();
		break;
	case kServerListTypeSimple:
		break;
	case kServerListTypeMaster:
		break;
	}
}

void ServerList::LoadFavorites()
{
	ifstream in;
	char buf[300];
	long curGame;
	long index;

	in.open("QFinder.fav", ios_base::in);
	if (!in.is_open())
	{
		// keine Favoriten Datei
		for (curGame = 1; curGame <= kGameCount; curGame++)
		{
			BCPort newPort(curGame, gGames[curGame].fStdPort);
			gQFinder->fBCPorts.insert(newPort);
		}
		fHasChanged = true;
		return;
	}
	curGame = 1;
	while (in.getline(buf, 300))
	{
		if (strlen(buf) < 2)
			continue;
		if (isalnum(buf[0]))
		{
			char strID[5];
			strID[0] = buf[0];

			long posBuf = 1;
			while (isalnum(buf[posBuf]) && posBuf < 4)
			{
				strID[posBuf] = buf[posBuf];
				posBuf++;
			}
			strID[posBuf] = 0;
			for (index = 1; index <= kGameCount; index++)
			{
				long length = gGames[index].fID.length();
				if (gGames[index].fID.compare(strID) == 0)
				{
					curGame = index;
					break;
				}
			}
		}

		if (strlen(buf) < 7)
			continue;
		if (strncmp(buf, "BROADCAST", 9) == 0)
		{
			if (buf[9] = ':')
			{
				if (isdigit(buf[10]))
				{
					char strPort[6];
					strPort[0] = buf[10];

					long posBuf = 11;
					long posPort = 1;
					while (isdigit(buf[posBuf]) && posPort < 5)
					{
						strPort[posPort] = buf[posBuf];
						posBuf++;
						posPort++;
					}
					strPort[posPort] = 0;
					BCPort newPort(curGame, atoi(strPort));
					gQFinder->fBCPorts.insert(newPort);
				}
			}
			continue;
		}
		if (isdigit(buf[0]))
		{
			char strIP[16];
			strIP[0] = buf[0];

			long posBuf = 1;
			while ((isdigit(buf[posBuf]) || buf[posBuf] == '.') && posBuf < 15)
			{
				strIP[posBuf] = buf[posBuf];
				posBuf++;
			}
			strIP[posBuf] = 0;
			unsigned long ip = ntohl(inet_addr(strIP));
			if (ip == INADDR_NONE)
				continue;
			unsigned short port = gGames[curGame].fStdPort;
			if (buf[posBuf] = ':')
			{
				posBuf++;
				if (isdigit(buf[posBuf]))
				{
					char strPort[6];
					strPort[0] = buf[posBuf];

					long posPort = 1;
					posBuf++;
					while (isdigit(buf[posBuf]) && posPort < 5)
					{
						strPort[posPort] = buf[posBuf];
						posBuf++;
						posPort++;
					}
					strPort[posPort] = 0;
					port = atoi(strPort);
				}
			}
			string desc = "";
			char *ptr = strchr(buf + posBuf, ';');
			if (ptr != NULL)
			{
				posBuf = ptr - buf;
				posBuf++;
				while (posBuf < (long) strlen(buf))
				{
					desc += buf[posBuf];
					posBuf++;
				}
			}
			GameServer newServer(ip, port, port);
			newServer.fGame = curGame;
			newServer.fDescription = desc;
			newServer.fListID = fID;
			fServers.insert(newServer);
		}
	}
	in.close();
}

void ServerList::SaveFavorites()
{
	ofstream out;
	long game;
	set<BCPort>::iterator itPort;
	ServerList *svrList;
	set<GameServer>::iterator itServer;
	string line;
	char port[10];

	if (!fHasChanged)
		return;
	out.open("QFinder.fav", ios_base::out);
	if (!out.is_open())
	{
		return;
	}
	line = ";Favorites List\n";
	out.write(line.c_str(), line.length());
	for (game = 1; game <= kGameCount; game++)
	{
		line = gGames[game].fID + " ;" + gGames[game].fTitle + "\n";
		out.write(line.c_str(), line.length());
		for (itPort = gQFinder->fBCPorts.begin();
			 itPort != gQFinder->fBCPorts.end();
			 itPort++)
		{
			if (itPort->fGame == game)
			{
				sprintf(port, "%u", itPort->fPort);
				line = "BROADCAST:";
				line += port;
				line += "\n";
				out.write(line.c_str(), line.length());
			}
		}
		svrList = gQFinder->GetServerList(gQFinder->fFavoritesID);
		for (itServer = svrList->fServers.begin();
			 itServer != svrList->fServers.end();
			 itServer++)
		{
			if (itServer->fGame == game)
			{
				line = itServer->fIPStr + ":" + itServer->fQueryPortStr;
				int len = 22 - line.length();
				for (int i = 0; i < len; i++)
					line += " ";
				line += ";";
				line += itServer->fDescription;
				line += "\n";
				out.write(line.c_str(), line.length());
			}
		}
	}
	out.close();
}
