/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "ServerListView.h"
#include "QFinder.h"
#include "MainWindow.h"

int CALLBACK ServerListView_CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);

ServerListView::ServerListView()
: ListView()
{
}

ServerListView::~ServerListView()
{
}

void ServerListView::Create(HWND hParent)
{
	ListView::Create(hParent, IDC_SERVER_LISTVIEW);
	SetStyleFullRowSelect();
	fImageList = ImageList_LoadImage(gQFinder->fInstance, MAKEINTRESOURCE (IDB_GAMEICONS_SMALL_16M), 16, 0, CLR_NONE, IMAGE_BITMAP, LR_CREATEDIBSECTION);
	SetImageList(fImageList);
	int i;
	int j;

	for (i = 0; i < (int) gQFinder->fSettings.fMainWindow.fServerColumns.size(); i++)
	{
		if (gQFinder->fSettings.fMainWindow.fServerColumns[i].fVisible)
		{
			fColumns.push_back(gQFinder->fSettings.fMainWindow.fServerColumns[i]);
		}
	}

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		for (j = 0; j < (int) fColumns.size(); j++)
		{
			if (fColumns[j].fPosition == i)
				break;
		}
		AddColumn(fColumns[j].fPosition, fColumns[j].fTitle, fColumns[j].fFormat, fColumns[j].fWidth, fColumns[j].fPosition);
	}
}

void ServerListView::Rebuild()
{
	int i;
	int j;

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		if (fColumns[i].fVisible)
			DeleteColumn(0);
	}
	fColumns.clear();

	for (i = 0; i < (int) gQFinder->fSettings.fMainWindow.fServerColumns.size(); i++)
	{
		if (gQFinder->fSettings.fMainWindow.fServerColumns[i].fVisible)
		{
			fColumns.push_back(gQFinder->fSettings.fMainWindow.fServerColumns[i]);
		}
	}

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		for (j = 0; j < (int) fColumns.size(); j++)
		{
			if (fColumns[j].fPosition == i)
				break;
		}
		AddColumn(fColumns[j].fPosition, fColumns[j].fTitle, fColumns[j].fFormat, fColumns[j].fWidth, fColumns[j].fPosition);
	}
}

void ServerListView::AddServer (GameServer *server)
{
	vector<string> itemStrings;
	string strItem;
	int item;
	long cnt;
	long colItem;
	char szHlp[16];
	
	strItem = server->fHostName;
	itemStrings.push_back(strItem);
	strItem = itoa(server->fPing, szHlp, 10);
	itemStrings.push_back(strItem);
	strItem = itoa(server->fCntPlayers, szHlp, 10);
	strItem += "/";
	strItem += itoa(server->fMaxPlayers, szHlp, 10);
	itemStrings.push_back(strItem);
	strItem = server->fMapName;
	itemStrings.push_back(strItem);
	strItem = server->fGameDir;
	itemStrings.push_back(strItem);
	strItem = server->fGameTypeStr;
	itemStrings.push_back(strItem);
	strItem = server->fIPStr;
	itemStrings.push_back(strItem);
	strItem = server->fPortStr;
	itemStrings.push_back(strItem);

	colItem = 0;
	for (cnt = 0; cnt < (int) fColumns.size(); cnt++)
	{
		if (fColumns[cnt].fVisible && fColumns[cnt].fPosition == 0)
		{
			colItem = cnt;
			break;
		}
	}
	item = FindServer(server);
	if (item != -1)
	{
		DeleteItem(item);
	}
	item = AddItem(itemStrings[colItem], server->fGame, (LPARAM) server);
	for (cnt = 0; cnt < (int) itemStrings.size(); cnt++)
	{
		if (cnt == colItem)
			continue;
		AddSubItem(itemStrings[cnt], item, fColumns[cnt].fPosition);
	}
	Sort();
}

long ServerListView::FindServer(GameServer *server)
{
	long count;
	long index;

	count = GetItemCount();
	for (index = 0; index < count; index++)
	{
		LVITEM item;

		item.iItem = index;
		item.iSubItem = 0;
		item.mask = LVIF_IMAGE | LVIF_PARAM;
		if (GetItem(&item))
		{
			GameServer *paramServer = (GameServer *) item.lParam;
			if (paramServer->fIP == server->fIP &&
				paramServer->fQueryPort == server->fQueryPort &&
				paramServer->fGame == server->fGame)
			{
				return index;
			}
		}
	}
	return -1;
}

GameServer *ServerListView::GetServer(long index)
{
	LVITEM item;
	item.iItem = index;
	item.iSubItem = 0;
	item.mask = LVIF_PARAM;
	if (GetItem(&item))
		return (GameServer *) item.lParam;
	else
		return false;
}

void ServerListView::Sort()
{
	vector<COLUMNINFO>::iterator itColumn;
	bool sorted = false;
	long subItem;

	for (itColumn = gQFinder->fSettings.fMainWindow.fServerColumns.begin();
		 itColumn != gQFinder->fSettings.fMainWindow.fServerColumns.end();
		 itColumn++)
	{
		if (itColumn->fSort != kColumnSortNone)
		{
			subItem = itColumn->fPosition;
			sorted = true;
			break;
		}
	}
	if (!sorted)
	{
		subItem = 0;
		for (itColumn = gQFinder->fSettings.fMainWindow.fServerColumns.begin();
			itColumn != gQFinder->fSettings.fMainWindow.fServerColumns.end();
			itColumn++)
		{
			if (itColumn->fPosition == 0)
			{
				itColumn->fSort = kColumnSortAscending;
				break;
			}
		}
	}
	SortItems(ServerListView_CompareFunc, subItem);
}

LRESULT ServerListView::OnNotify (LPARAM lParam)
{
	NMLISTVIEW *nmListView = (NMLISTVIEW *) lParam;

	switch (nmListView->hdr.code)
	{
	case LVN_COLUMNCLICK:
		{
			vector<COLUMNINFO>::iterator itColumn;
			bool newCol = false;
			for (itColumn = gQFinder->fSettings.fMainWindow.fServerColumns.begin();
				 itColumn != gQFinder->fSettings.fMainWindow.fServerColumns.end();
				 itColumn++)
			{
				if (itColumn->fSort != kColumnSortNone)
				{
					if (itColumn->fPosition != nmListView->iSubItem)
					{
						itColumn->fSort = kColumnSortNone;
						newCol = true;
						break;
					}
					else
					{
						if (itColumn->fSort == kColumnSortAscending)
							itColumn->fSort = kColumnSortDescending;
						else
							itColumn->fSort = kColumnSortAscending;
					}
				}
			}
			if (newCol)
			{
				for (itColumn = gQFinder->fSettings.fMainWindow.fServerColumns.begin();
					 itColumn != gQFinder->fSettings.fMainWindow.fServerColumns.end();
					 itColumn++)
				{
					if (itColumn->fPosition == nmListView->iSubItem)
					{
						itColumn->fSort = kColumnSortAscending;
						break;
					}
				}
			}
			SortItems(ServerListView_CompareFunc, nmListView->iSubItem);
		}
		break;
	case LVN_ITEMCHANGED:
		if (nmListView->uNewState == 3)
		{
			LVITEM item;
			item.iItem = nmListView->iItem;
			item.iSubItem = 0;
			item.mask = LVIF_IMAGE | LVIF_PARAM;
			if (GetItem(&item))
			{
				GameServer *server = (GameServer *) item.lParam;
				gMainWindow->SetCurrentServer(server);
				gMainWindow->SetOldServer(server);
				PlayerListView *playerList = gMainWindow->GetPlayerListView();
				playerList->DeleteAllItems();
				vector<Player>::iterator itPlayer;
				for (itPlayer = server->fPlayers.begin();
					 itPlayer != server->fPlayers.end();
					 itPlayer++)
				{
					playerList->AddPlayer(itPlayer.operator ->());
				}
				playerList->Sort();
				VariableListView *varList = gMainWindow->GetVariableListView();
				varList->DeleteAllItems();
				list<StringVariable>::iterator itVariable;
				for (itVariable = server->fVariables.begin();
					 itVariable != server->fVariables.end();
					 itVariable++)
				{
					varList->AddVariable(itVariable.operator ->());
				}
				varList->Sort();
			}
		}
		break;
	}
	return 0;
}

int CALLBACK ServerListView_CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	GameServer *server1 = (GameServer *) lParam1;
	GameServer *server2 = (GameServer *) lParam2;
	long subItem = lParamSort;
	int retVal = 0;
	int sortpos1 = 0, sortpos2 = 0;

	vector<COLUMNINFO>::iterator itColumn;
	long ixCol = 0;
	for (itColumn = gQFinder->fSettings.fMainWindow.fServerColumns.begin();
		 itColumn != gQFinder->fSettings.fMainWindow.fServerColumns.end();
		 itColumn++)
	{
		if (itColumn->fPosition == subItem)
		{
			break;
		}
		ixCol++;
	}
	switch (ixCol)
	{
	case kColumnServerHostName:
		for (int i = 1; i <= kGameCount; i++)
		{
			if (gQFinder->fSettings.fGame.fGameSort[i] == server1->fGame)
				sortpos1 = i;
			if (gQFinder->fSettings.fGame.fGameSort[i] == server2->fGame)
				sortpos2 = i;
		}

		if (sortpos1 < sortpos2)
			retVal = -1;
		else
			if (sortpos1 > sortpos2)
				retVal = 1;
			else
				retVal = server1->fHostName.compare(server2->fHostName);
		break;
	case kColumnServerPing:
		if (server1->fPing < server2->fPing)
			retVal = -1;
		else
			if (server1->fPing > server2->fPing)
				retVal = 1;
			else
				retVal = 0;
		break;
	case kColumnServerPlayers:
		if (server1->fCntPlayers < server2->fCntPlayers)
			retVal = -1;
		else
			if (server1->fCntPlayers > server2->fCntPlayers)
				retVal = 1;
			else
				retVal = 0;
		break;
	case kColumnServerMapName:
		retVal = server1->fMapName.compare(server2->fMapName);
		break;
	case kColumnServerGameDir:
		retVal = server1->fGameDir.compare(server2->fGameDir);
		break;
	case kColumnServerGameType:
		retVal = server1->fGameTypeStr.compare(server2->fGameTypeStr);
		break;
	case kColumnServerIP:
		if (server1->fIP < server2->fIP)
			retVal = -1;
		else
			if (server1->fIP > server2->fIP)
				retVal = 1;
			else
				retVal = 0;
		break;
	case kColumnServerPort:
		if (server1->fPort < server2->fPort)
			retVal = -1;
		else
			if (server1->fPort > server2->fPort)
				retVal = 1;
			else
				retVal = 0;
		break;
	}
	if (itColumn->fSort == kColumnSortAscending)
		return retVal;
	else
		return -retVal;
}
