/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#ifndef _SETTINGS_H_
#define _SETTINGS_H_

#include "Game.h"

const long kWindowModeNormal = 0;
const long kWindowModeMaximized = 1;
const long kWindowModeMinimized = 2;

const long kLayout_Standard = 0;
const long kLayout_Mini = 1;

const long kLayout_TreeViewShort = 0;
const long kLayout_TreeViewLong = 1;

const long kLayout_PlayerLeft = 0;
const long kLayout_PlayerRight = 1;

const long kMinimizeModeNormal = 0;
const long kMinimizeModeTray = 1;

const long kToolBarTextNone = 0;
const long kToolBarTextBelow = 1;
const long kToolBarTextRight = 2;

const long kColumnSortNone = 0;
const long kColumnSortAscending = 1;
const long kColumnSortDescending = 2;

const long kColumnServerHostName = 0;
const long kColumnServerPing = 1;
const long kColumnServerPlayers = 2;
const long kColumnServerMapName = 3;
const long kColumnServerGameDir = 4;
const long kColumnServerGameType = 5;
const long kColumnServerIP = 6;
const long kColumnServerPort = 7;

const long kColumnPlayerName = 0;
const long kColumnPlayerFrags = 1;
const long kColumnPlayerPing = 2;

const long kColumnVariableName = 0;
const long kColumnVariableValue = 1;

const long kChangedLayout = 1;
const long kChangedToolBar = 2;
const long kChangedServerList = 4;
const long kChangedPlayerList = 8;
const long kChangedVariableList = 16;
const long kChangedGameList = 32;

typedef struct
{
	char fVisible;
	char fPosition;
	short fWidth;
	char fSort;
	long fFormat;
	string fTitle;
	string fID;
	long fSaveToRegistry;
} COLUMNINFO;

typedef struct
{
	char fVisible;
	char fPosition;
	string fPath;
	string fExe;
	string fParameters;
	long fSaveToRegistry;
} GAMEINFO;

class Settings
{
public:
	void Init();
	void Load();
	void Check();
	void Save();
	void SetChanged(Settings& old);
	Settings& operator= (Settings& other);

	long fRegVersion;
	struct WINDOW {
		long fMode;
		long fLeft;
		long fTop;
		long fWidth;
		long fHeight;
		long fTVWidth;
		long fLVHeight;
		long fLVWidth;
		long fLayout;
		long fLayoutParameter1;
		long fLayoutParameter2;
		long fMinimizeMode;
		long fToolBarTextMode;
		vector <COLUMNINFO> fServerColumns;
		vector <COLUMNINFO> fPlayerColumns;
		vector <COLUMNINFO> fVariableColumns;
	} fMainWindow;
	struct GAME {
		string fStandard;
		string fLastSelected;
		long fGameSort[kGameCount + 1];
		vector <GAMEINFO> fInfos;
	} fGame;
	struct HOTKEYS {
		bool minimizeActive;
		short minimizeId;
		unsigned char minimizeModifier;
		unsigned char minimizeVirtualKey;
		bool minimizeRefresh;
	} fHotkeys;
	long fChanged;
private:
	void CheckColumns(vector<COLUMNINFO> &columns);

	bool LoadSetting(string folder, string name, string &value);
	bool LoadSetting(string folder, string name, long &value);
	bool LoadSetting(string folder, string name, unsigned char *value, long &length);
	
	void SaveSetting(string folder, string name, string value);
	void SaveSetting(string folder, string name, long value);
	void SaveSetting(string folder, string name, unsigned char *value, long length);
};

#endif
