/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "SocketWindow.h"
#include "QFinder.h"
#include "MainWindow.h"
#include "ServerStatus.h"

SocketWindow::SocketWindow(MainWindow *mainWnd)
{
	WNDCLASS wndClass;
	
	wndClass.style         = 0;
	wndClass.lpfnWndProc   = SocketWndProc;
	wndClass.cbClsExtra    = 0;
	wndClass.cbWndExtra    = 0;
	wndClass.hInstance     = gQFinder->fInstance;
	wndClass.hIcon         = NULL;
	wndClass.hCursor       = NULL;
	wndClass.hbrBackground = 0;
	wndClass.lpszMenuName  = 0;
	wndClass.lpszClassName = "SocketWindow";
	
	if (!RegisterClass(&wndClass))
	{
		return;
	}

	fWindow = CreateWindow("SocketWindow",	// Name der Fensterklasse
						   "SocketWindow",	// Fenstertitel
						   0,	// Fensterstil
						   CW_USEDEFAULT,	// X-Position des Fensters
						   CW_USEDEFAULT,	// Y-Position des Fensters
						   CW_USEDEFAULT,	// Fensterbreite
						   CW_USEDEFAULT,	// Fensterhhe
						   NULL,			// bergeordnetes Fenster
						   NULL,			// Men
						   gQFinder->fInstance,		// Programm-Kopiezhler (Programm-ID)
						   NULL);			// zustzliche Parameter

	fMainWnd = mainWnd;
}

SocketWindow::~SocketWindow()
{
}

LRESULT SocketWindow::WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	vector<struct SOCKETEVENT>::iterator i;
	char buf[10000];
	struct sockaddr_in fromAddr;
	int fromLen = sizeof (sockaddr);
	int len;

	switch (message)
	{
	case WM_USER:
		for (i = fSockets.begin(); i != fSockets.end(); i++)
		{
			if (i->fSocket == wParam)
			{
				switch (WSAGETSELECTEVENT(lParam))
				{
				case FD_READ:
					len = recvfrom (i->fSocket, buf, 10000, 0, (sockaddr *) &fromAddr, &fromLen);
					ServerStatus * newStatus = new ServerStatus();
					newStatus->SetAddress(ntohl(fromAddr.sin_addr.S_un.S_addr), ntohs(fromAddr.sin_port));
					if (len == SOCKET_ERROR)
					{
						newStatus->fError = WSAGetLastError();
						newStatus->fStatus = "";
						break;
					}
					else
					{
						newStatus->fError = 0;
						newStatus->fPing = GetTickCount() - i->fStartTicks;
						newStatus->fStatus.assign(buf, len);
					}
					fMainWnd->SendMsg(i->fMessage, (WPARAM) newStatus, i->fSocket);
					break;
				}
			}
		}
		break;
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}

void SocketWindow::AddSocketEvent(SOCKET newSocket, long event, int message)
{
	struct SOCKETEVENT newEvent;
	vector<struct SOCKETEVENT>::iterator i;

	for (i = fSockets.begin(); i != fSockets.end(); i++)
	{
		if (i->fSocket == newSocket)
		{
			fSockets.erase(i);
			break;
		}
	}

	if (WSAAsyncSelect(newSocket, fWindow, WM_USER, event) == 0)
	{
		newEvent.fSocket = newSocket;
		newEvent.fEvent = event;
		newEvent.fMessage = message;
		newEvent.fStartTicks = GetTickCount();
		fSockets.push_back(newEvent);
	}
}

void SocketWindow::RemoveSocketEvent(SOCKET existingSocket)
{
	vector<struct SOCKETEVENT>::iterator i;

	for (i = fSockets.begin(); i != fSockets.end(); i++)
	{
		if (i->fSocket == existingSocket)
		{
			fSockets.erase(i);
			break;
		}
	}
}

LRESULT APIENTRY SocketWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	return gSocketWindow->WindowProc(hWnd, message, wParam, lParam);
}

SocketWindow *gSocketWindow;