/*
 * QFinder - A game server browser for Windows
 *
 * Copyright (C) 1999,2004 Andreas Regel <andreas.regel@powarman.de>
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 */

#include "StdAfx.h"
#include "VariableListView.h"
#include "QFinder.h"

int CALLBACK VariableListView_CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);

VariableListView::VariableListView()
: ListView()
{
}

VariableListView::~VariableListView()
{
}

void VariableListView::Create(HWND hParent)
{
	ListView::Create(hParent, IDC_VARIABLE_LISTVIEW);
	SetStyleFullRowSelect();
	COLUMNINFO newCol;
	int i;
	int j;

	for (i = 0; i < (int) gQFinder->fSettings.fMainWindow.fVariableColumns.size(); i++)
	{
		newCol.fVisible = gQFinder->fSettings.fMainWindow.fVariableColumns[i].fVisible;
		newCol.fPosition = gQFinder->fSettings.fMainWindow.fVariableColumns[i].fPosition;
		newCol.fWidth = gQFinder->fSettings.fMainWindow.fVariableColumns[i].fWidth;
		newCol.fSort = gQFinder->fSettings.fMainWindow.fVariableColumns[i].fSort;
		newCol.fFormat = gQFinder->fSettings.fMainWindow.fVariableColumns[i].fFormat;
		newCol.fTitle = gQFinder->fSettings.fMainWindow.fVariableColumns[i].fTitle;
		fColumns.push_back(newCol);
	}

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		for (j = 0; j < (int) fColumns.size(); j++)
		{
			if (fColumns[j].fPosition == i)
				break;
		}
		AddColumn(fColumns[j].fPosition, fColumns[j].fTitle, fColumns[j].fFormat, fColumns[j].fWidth, fColumns[j].fPosition);
	}
}

void VariableListView::Rebuild()
{
	int i;
	int j;

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		if (fColumns[i].fVisible)
			DeleteColumn(0);
	}
	fColumns.clear();

	for (i = 0; i < (int) gQFinder->fSettings.fMainWindow.fVariableColumns.size(); i++)
	{
		if (gQFinder->fSettings.fMainWindow.fVariableColumns[i].fVisible)
		{
			fColumns.push_back(gQFinder->fSettings.fMainWindow.fVariableColumns[i]);
		}
	}

	for (i = 0; i < (int) fColumns.size(); i++)
	{
		for (j = 0; j < (int) fColumns.size(); j++)
		{
			if (fColumns[j].fPosition == i)
				break;
		}
		AddColumn(fColumns[j].fPosition, fColumns[j].fTitle, fColumns[j].fFormat, fColumns[j].fWidth, fColumns[j].fPosition);
	}
}

void VariableListView::AddVariable (StringVariable *variable)
{
	vector<string> itemStrings;
	string strItem;
	int item;
	long cnt;
	long colItem;

	strItem = variable->fName;
	itemStrings.push_back(strItem);
	strItem = variable->fValue;
	itemStrings.push_back(strItem);

	colItem = 0;
	for (cnt = 0; cnt < (int) fColumns.size(); cnt++)
	{
		if (fColumns[cnt].fVisible && fColumns[cnt].fPosition == 0)
		{
			colItem = cnt;
			break;
		}
	}
	item = AddItem(itemStrings[colItem], 0, (LPARAM) variable);
	for (cnt = 0; cnt < (int) itemStrings.size(); cnt++)
	{
		if (cnt == colItem)
			continue;
		AddSubItem(itemStrings[cnt], item, fColumns[cnt].fPosition);
	}
}

void VariableListView::Sort()
{
	vector<COLUMNINFO>::iterator itColumn;
	bool sorted = false;
	long subItem;

	for (itColumn = gQFinder->fSettings.fMainWindow.fVariableColumns.begin();
		 itColumn != gQFinder->fSettings.fMainWindow.fVariableColumns.end();
		 itColumn++)
	{
		if (itColumn->fSort != kColumnSortNone)
		{
			subItem = itColumn->fPosition;
			sorted = true;
			break;
		}
	}
	if (!sorted)
	{
		subItem = 0;
		for (itColumn = gQFinder->fSettings.fMainWindow.fVariableColumns.begin();
			itColumn != gQFinder->fSettings.fMainWindow.fVariableColumns.end();
			itColumn++)
		{
			if (itColumn->fPosition == 0)
			{
				itColumn->fSort = kColumnSortAscending;
				break;
			}
		}
	}
	SortItems(VariableListView_CompareFunc, subItem);
}

LRESULT VariableListView::OnNotify (LPARAM lParam)
{
	NMLISTVIEW *nmListView = (NMLISTVIEW *) lParam;

	switch (nmListView->hdr.code)
	{
	case LVN_COLUMNCLICK:
		{
			vector<COLUMNINFO>::iterator itColumn;
			bool newCol = false;
			for (itColumn = gQFinder->fSettings.fMainWindow.fVariableColumns.begin();
				 itColumn != gQFinder->fSettings.fMainWindow.fVariableColumns.end();
				 itColumn++)
			{
				if (itColumn->fSort != kColumnSortNone)
				{
					if (itColumn->fPosition != nmListView->iSubItem)
					{
						itColumn->fSort = kColumnSortNone;
						newCol = true;
						break;
					}
					else
					{
						if (itColumn->fSort == kColumnSortAscending)
							itColumn->fSort = kColumnSortDescending;
						else
							itColumn->fSort = kColumnSortAscending;
					}
				}
			}
			if (newCol)
			{
				for (itColumn = gQFinder->fSettings.fMainWindow.fVariableColumns.begin();
					 itColumn != gQFinder->fSettings.fMainWindow.fVariableColumns.end();
					 itColumn++)
				{
					if (itColumn->fPosition == nmListView->iSubItem)
					{
						itColumn->fSort = kColumnSortAscending;
						break;
					}
				}
			}
			SortItems(VariableListView_CompareFunc, nmListView->iSubItem);
		}
		break;
	case LVN_ITEMCHANGED:
		if (nmListView->uNewState == 3)
		{
		}
		break;
	}
	return 0;
}

int CALLBACK VariableListView_CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	StringVariable *paramVar1 = (StringVariable *) lParam1;
	StringVariable *paramVar2 = (StringVariable *) lParam2;
	long subItem = lParamSort;
	int retVal = 0;
	vector<COLUMNINFO>::iterator itColumn;
	long ixCol = 0;

	for (itColumn = gQFinder->fSettings.fMainWindow.fVariableColumns.begin();
		 itColumn != gQFinder->fSettings.fMainWindow.fVariableColumns.end();
		 itColumn++)
	{
		if (itColumn->fPosition == subItem)
		{
			break;
		}
		ixCol++;
	}
	switch (ixCol)
	{
	case kColumnVariableName:
		retVal = paramVar1->fName.compare(paramVar2->fName);
		break;
	case kColumnVariableValue:
		retVal = paramVar1->fValue.compare(paramVar2->fValue);
		break;
	}
	if (itColumn->fSort == kColumnSortAscending)
		return retVal;
	else
		return -retVal;
}
